@@ -22,6 +22,7 @@
 #include "caf/config_option_adder.hpp"
 #include "caf/config_option_set.hpp"
 #include "caf/config_value.hpp"
+#include "caf/detail/set_thread_name.hpp"
 #include "caf/settings.hpp"
 #include "caf/string_algorithms.hpp"
 #include "caf/test/unit_test.hpp"
@@ -36,6 +37,7 @@ public:
 private:
   watchdog(int secs) {
     thread_ = std::thread{[=] {
+      caf::detail::set_thread_name("test.watchdog");
       auto tp = std::chrono::high_resolution_clock::now()
                 + std::chrono::seconds(secs);
       std::unique_lock<std::mutex> guard{mtx_};
@@ -148,6 +150,8 @@ bool check(test* parent, const char* file, size_t line, const char* expr,
         << term::blue << line << fill(line) << term::reset << expr << '\n';
     parent->fail(should_fail);
   }
+  engine::last_check_file(file);
+  engine::last_check_line(line);
   return result;
 }
 
