@@ -1,3 +1,5 @@
+#include <memory>
+
 #include "broker/internal/metric_collector.hh"
 
 #include "broker/internal/logger.hh"
@@ -295,22 +297,28 @@ metric_collector::instance(const std::string& endpoint_name, metric_view mv) {
     instance_ptr new_instance;
     switch (mv.type()) {
       case metric_type::int_counter:
-        new_instance.reset(new remote_counter<integer>(owned(labels), fptr));
+        new_instance = std::make_unique<remote_counter<integer>>(owned(labels),
+                                                                 fptr);
         break;
       case metric_type::dbl_counter:
-        new_instance.reset(new remote_counter<real>(owned(labels), fptr));
+        new_instance = std::make_unique<remote_counter<real>>(owned(labels),
+                                                              fptr);
         break;
       case metric_type::int_gauge:
-        new_instance.reset(new remote_gauge<integer>(owned(labels), fptr));
+        new_instance = std::make_unique<remote_gauge<integer>>(owned(labels),
+                                                               fptr);
         break;
       case metric_type::dbl_gauge:
-        new_instance.reset(new remote_gauge<real>(owned(labels), fptr));
+        new_instance = std::make_unique<remote_gauge<real>>(owned(labels),
+                                                            fptr);
         break;
       case metric_type::int_histogram:
-        new_instance.reset(new remote_histogram<integer>(owned(labels), fptr));
+        new_instance =
+          std::make_unique<remote_histogram<integer>>(owned(labels), fptr);
         break;
       case metric_type::dbl_histogram:
-        new_instance.reset(new remote_histogram<real>(owned(labels), fptr));
+        new_instance = std::make_unique<remote_histogram<real>>(owned(labels),
+                                                                fptr);
         break;
       default:
         return nullptr;
