@@ -12,15 +12,21 @@
 #include <hilti/base/util.h>
 #include <hilti/compiler/unit.h>
 
+#include <spicy/ast/detail/visitor.h>
 #include <spicy/global.h>
 #include <zeek-spicy/autogen/config.h>
-
-#include "debug.h"
+#include <zeek-spicy/compiler/debug.h>
 
 using namespace spicy::zeek;
 
 namespace builder = hilti::builder;
 
+#if SPICY_VERSION_NUMBER >= 10700
+static inline auto _linker_scope() { return hilti::builder::scope(); }
+#else
+static inline auto _linker_scope() { return hilti::builder::call("hilti::linker_scope", {}); }
+#endif
+
 // Small parsing helpers.
 
 using ParseError = std::runtime_error;
@@ -95,6 +101,36 @@ static hilti::ID extract_id(const std::string& chunk, size_t* i) {
     return hilti::ID(hilti::util::replace(id, "%", "0x25_"));
 }
 
+static hilti::Type extract_type(const std::string& chunk, size_t* i) {
+    eat_spaces(chunk, i);
+
+    // We currently only parse Spicy types that can appear in parameters of
+    // built-in hooks--which are not many.
+    auto token = extract_id(chunk, i);
+
+    if ( token == hilti::ID("string") )
+        return hilti::type::String();
+
+    if ( token == hilti::ID("uint64") )
+        return hilti::type::UnsignedInteger(64);
+
+    throw ParseError("mismatching type");
+}
+
+static hilti::type::function::Parameter extract_parameter(const std::string& chunk, size_t* i) {
+    eat_spaces(chunk, i);
+
+    auto id = extract_id(chunk, i);
+
+    if ( ! looking_at(chunk, *i, ":") )
+        throw ParseError("expected ':'");
+
+    eat_token(chunk, i, ":");
+
+    auto type = extract_type(chunk, i);
+    return builder::parameter(std::move(id), std::move(type));
+}
+
 static hilti::rt::filesystem::path extract_path(const std::string& chunk, size_t* i) {
     eat_spaces(chunk, i);
 
@@ -272,7 +308,10 @@ static std::vector<hilti::rt::Port> extract_ports(const std::string& chunk, size
     return result;
 }
 
-GlueCompiler::GlueCompiler(Driver* driver, int zeek_version) : _driver(driver), _zeek_version(zeek_version) {}
+void GlueCompiler::Init(Driver* driver, int zeek_version) {
+    _driver = driver;
+    _zeek_version = zeek_version;
+}
 
 GlueCompiler::~GlueCompiler() {}
 
@@ -344,7 +383,7 @@ hilti::Result<std::string> GlueCompiler::getNextEvtBlock(std::istream& in, int*
 }
 
 void GlueCompiler::preprocessEvtFile(hilti::rt::filesystem::path& path, std::istream& in, std::ostream& out) {
-    hilti::util::SourceCodePreprocessor pp({{"ZEEK_VERSION", _zeek_version}});
+    hilti::util::SourceCodePreprocessor pp({{"ZEEK_VERSION", *_zeek_version}});
     int lineno = 0;
 
     std::string line;
@@ -380,6 +419,8 @@ void GlueCompiler::preprocessEvtFile(hilti::rt::filesystem::path& path, std::ist
 }
 
 bool GlueCompiler::loadEvtFile(hilti::rt::filesystem::path& path) {
+    assert(_zeek_version);
+
     std::ifstream in(path);
 
     if ( ! in ) {
@@ -413,20 +454,20 @@ bool GlueCompiler::loadEvtFile(hilti::rt::filesystem::path& path) {
 
             if ( looking_at(*chunk, 0, "protocol") ) {
                 auto a = parseProtocolAnalyzer(*chunk);
-                _protocol_analyzers.push_back(a);
                 ZEEK_DEBUG(hilti::util::fmt("  Got protocol analyzer definition for %s", a.name));
+                _protocol_analyzers.push_back(a);
             }
 
             else if ( looking_at(*chunk, 0, "file") ) {
                 auto a = parseFileAnalyzer(*chunk);
-                _file_analyzers.push_back(a);
                 ZEEK_DEBUG(hilti::util::fmt("  Got file analyzer definition for %s", a.name));
+                _file_analyzers.push_back(a);
             }
 
             else if ( looking_at(*chunk, 0, "packet") ) {
                 auto a = parsePacketAnalyzer(*chunk);
-                _packet_analyzers.push_back(a);
                 ZEEK_DEBUG(hilti::util::fmt("  Got packet analyzer definition for %s", a.name));
+                _packet_analyzers.push_back(a);
             }
 
             else if ( looking_at(*chunk, 0, "on") ) {
@@ -454,8 +495,27 @@ bool GlueCompiler::loadEvtFile(hilti::rt::filesystem::path& path) {
                 _imports.emplace_back(hilti::ID(module), std::move(scope));
             }
 
+            else if ( looking_at(*chunk, 0, "export") ) {
+                size_t i = 0;
+                eat_token(*chunk, &i, "export");
+
+                hilti::ID spicy_id = extract_id(*chunk, &i);
+                hilti::ID zeek_id = spicy_id;
+
+                if ( looking_at(*chunk, i, "as") ) {
+                    eat_token(*chunk, &i, "as");
+                    zeek_id = extract_id(*chunk, &i);
+                }
+
+                eat_spaces(*chunk, &i);
+                if ( ! looking_at(*chunk, i, ";") )
+                    throw ParseError("syntax error in export");
+
+                _exports.emplace_back(std::move(spicy_id), std::move(zeek_id), _locations.back());
+            }
+
             else
-                throw ParseError("expected 'import', '{file,protocol} analyzer', or 'on'");
+                throw ParseError("expected 'import', 'export', '{file,packet,protocol} analyzer', or 'on'");
 
             _locations.pop_back();
         }
@@ -488,7 +548,7 @@ glue::ProtocolAnalyzer GlueCompiler::parseProtocolAnalyzer(const std::string& ch
 
     eat_token(chunk, &i, "protocol");
     eat_token(chunk, &i, "analyzer");
-    a.name = hilti::util::replace(extract_id(chunk, &i), "::", "_");
+    a.name = extract_id(chunk, &i).str();
 
     eat_token(chunk, &i, "over");
 
@@ -593,7 +653,7 @@ glue::FileAnalyzer GlueCompiler::parseFileAnalyzer(const std::string& chunk) {
 
     eat_token(chunk, &i, "file");
     eat_token(chunk, &i, "analyzer");
-    a.name = hilti::util::replace(extract_id(chunk, &i).str(), "::", "_");
+    a.name = extract_id(chunk, &i).str();
 
     eat_token(chunk, &i, ":");
 
@@ -611,9 +671,6 @@ glue::FileAnalyzer GlueCompiler::parseFileAnalyzer(const std::string& chunk) {
         }
 
         else if ( looking_at(chunk, i, "replaces") ) {
-            if ( _zeek_version < 40100 )
-                throw ParseError("file analyzer replacement requires Zeek 4.1+");
-
             eat_token(chunk, &i, "replaces");
             a.replaces = extract_id(chunk, &i);
         }
@@ -638,9 +695,6 @@ glue::PacketAnalyzer GlueCompiler::parsePacketAnalyzer(const std::string& chunk)
 
     eat_token(chunk, &i, "packet");
     eat_token(chunk, &i, "analyzer");
-
-    // We don't normalize the name here so that the user can address
-    // it with the expected spelling.
     a.name = extract_id(chunk, &i).str();
 
     eat_token(chunk, &i, ":");
@@ -652,6 +706,14 @@ glue::PacketAnalyzer GlueCompiler::parsePacketAnalyzer(const std::string& chunk)
             a.unit_name = extract_id(chunk, &i);
         }
 
+        else if ( looking_at(chunk, i, "replaces") ) {
+            if ( *_zeek_version < 50200 )
+                throw ParseError("packet analyzer replacement requires Zeek 5.2+");
+
+            eat_token(chunk, &i, "replaces");
+            a.replaces = extract_id(chunk, &i);
+        }
+
         else
             throw ParseError("unexpect token");
 
@@ -677,6 +739,24 @@ glue::Event GlueCompiler::parseEvent(const std::string& chunk) {
     eat_token(chunk, &i, "on");
     ev.path = extract_id(chunk, &i);
 
+    if ( looking_at(chunk, i, "(") ) {
+        eat_token(chunk, &i, "(");
+
+        if ( ! looking_at(chunk, i, ")") ) {
+            while ( true ) {
+                auto param = extract_parameter(chunk, &i);
+                ev.parameters.push_back(std::move(param));
+
+                if ( looking_at(chunk, i, ")") )
+                    break;
+
+                eat_token(chunk, &i, ",");
+            }
+        }
+
+        eat_token(chunk, &i, ")");
+    }
+
     if ( looking_at(chunk, i, "if") ) {
         eat_token(chunk, &i, "if");
         eat_token(chunk, &i, "(");
@@ -727,6 +807,8 @@ glue::Event GlueCompiler::parseEvent(const std::string& chunk) {
 }
 
 bool GlueCompiler::compile() {
+    assert(_driver);
+
     auto init_module = hilti::Module(hilti::ID("spicy_init"));
 
     auto import_ = hilti::builder::import(ID("zeek_rt"), ".hlt");
@@ -735,8 +817,20 @@ bool GlueCompiler::compile() {
     import_ = hilti::builder::import(ID("hilti"), ".hlt");
     init_module.add(std::move(import_));
 
+    // Declare the plugin's version function.
+    auto cxxname = hilti::AttributeSet(
+        {hilti::Attribute("&cxxname", builder::string(ZEEK_SPICY_PLUGIN_VERSION_FUNCTION_AS_STRING)),
+         hilti::Attribute("&have_prototype", builder::bool_(true))});
+    auto version_function =
+        hilti::builder::function("zeek_spicy_plugin_version", type::String(), {}, type::function::Flavor::Standard,
+                                 declaration::Linkage::Public, function::CallingConvention::Standard, cxxname);
+    init_module.add(std::move(version_function));
+
     auto preinit_body = hilti::builder::Builder(_driver->context());
 
+    // Call the plugin's version function.
+    preinit_body.addCall("zeek_spicy_plugin_version", {});
+
     for ( auto&& [id, m] : _spicy_modules )
         m->spicy_module = hilti::Module(hilti::ID(hilti::util::fmt("spicy_hooks_%s", id)));
 
@@ -747,28 +841,31 @@ bool GlueCompiler::compile() {
         ZEEK_DEBUG(hilti::util::fmt("Adding protocol analyzer '%s'", a.name));
 
         if ( a.unit_name_orig ) {
-            if ( auto ui = _driver->lookupUnit(a.unit_name_orig) )
+            if ( auto ui = _driver->lookupType<spicy::type::Unit>(a.unit_name_orig) )
                 a.unit_orig = *ui;
             else {
-                hilti::logger().error(
-                    hilti::util::fmt("unknown unit type %s with protocol analyzer %s", a.unit_name_orig, a.name));
+                hilti::logger().error(hilti::util::fmt("error with protocol analyzer %s: %s", a.name, ui.error()));
                 return false;
             }
         }
 
         if ( a.unit_name_resp ) {
-            if ( auto ui = _driver->lookupUnit(a.unit_name_resp) )
+            if ( auto ui = _driver->lookupType<spicy::type::Unit>(a.unit_name_resp) )
                 a.unit_resp = *ui;
             else {
-                hilti::logger().error(
-                    hilti::util::fmt("unknown unit type %s with protocol analyzer %s", a.unit_name_resp, a.name));
+                hilti::logger().error(hilti::util::fmt("error with protocol analyzer %s: %s", a.name, ui.error()));
                 return false;
             }
         }
 
-        hilti::ID protocol;
+#if SPICY_VERSION_NUMBER >= 10700
+        auto proto = a.protocol.value();
+#else
+        auto proto = a.protocol;
+#endif
 
-        switch ( a.protocol ) {
+        hilti::ID protocol;
+        switch ( proto ) {
             case hilti::rt::Protocol::TCP: protocol = hilti::ID("hilti::Protocol::TCP"); break;
             case hilti::rt::Protocol::UDP: protocol = hilti::ID("hilti::Protocol::UDP"); break;
             default: hilti::logger().internalError("unexpected protocol");
@@ -778,18 +875,17 @@ bool GlueCompiler::compile() {
                              {builder::string(a.name), builder::id(protocol),
                               builder::vector(hilti::util::transform(a.ports, [](auto p) { return builder::port(p); })),
                               builder::string(a.unit_name_orig), builder::string(a.unit_name_resp),
-                              builder::string(a.replaces), builder::call("hilti::linker_scope", {})});
+                              builder::string(a.replaces), _linker_scope()});
     }
 
     for ( auto& a : _file_analyzers ) {
         ZEEK_DEBUG(hilti::util::fmt("Adding file analyzer '%s'", a.name));
 
         if ( a.unit_name ) {
-            if ( auto ui = _driver->lookupUnit(a.unit_name) )
+            if ( auto ui = _driver->lookupType<spicy::type::Unit>(a.unit_name) )
                 a.unit = *ui;
             else {
-                hilti::logger().error(
-                    hilti::util::fmt("unknown unit type %s with file analyzer %s", a.unit_name, a.name));
+                hilti::logger().error(hilti::util::fmt("error with file analyzer %s: %s", a.name, ui.error()));
                 return false;
             }
         }
@@ -798,37 +894,26 @@ bool GlueCompiler::compile() {
                              {builder::string(a.name),
                               builder::vector(
                                   hilti::util::transform(a.mime_types, [](auto m) { return builder::string(m); })),
-                              builder::string(a.unit_name), builder::string(a.replaces),
-                              builder::call("hilti::linker_scope", {})});
+                              builder::string(a.unit_name), builder::string(a.replaces), _linker_scope()});
     }
 
     for ( auto& a : _packet_analyzers ) {
         ZEEK_DEBUG(hilti::util::fmt("Adding packet analyzer '%s'", a.name));
 
         if ( a.unit_name ) {
-            if ( auto ui = _driver->lookupUnit(a.unit_name) )
+            if ( auto ui = _driver->lookupType<spicy::type::Unit>(a.unit_name) )
                 a.unit = *ui;
             else {
-                hilti::logger().error(
-                    hilti::util::fmt("unknown unit type %s with packet analyzer %s", a.unit_name, a.name));
+                hilti::logger().error(hilti::util::fmt("error with packet analyzer %s: %s", a.name, ui.error()));
                 return false;
             }
         }
 
         preinit_body.addCall("zeek_rt::register_packet_analyzer",
-                             {builder::string(a.name), builder::string(a.unit_name),
-                              builder::call("hilti::linker_scope", {})});
+                             {builder::string(a.name), builder::string(a.unit_name), builder::string(a.replaces),
+                              _linker_scope()});
     }
 
-#if 0
-    // Check that our events align with what's defined on the Zeek side.
-    // TODO: See comment in header for CheckZeekEvent().
-    for ( auto&& ev : _events ) {
-        if ( ! CheckZeekEvent(ev) )
-            return false;
-    }
-#endif
-
     // Create the Spicy hooks and accessor functions.
     for ( auto&& ev : _events ) {
         if ( ! CreateSpicyHook(&ev) )
@@ -839,21 +924,16 @@ bool GlueCompiler::compile() {
     for ( auto&& ev : _events )
         preinit_body.addCall("zeek_rt::install_handler", {builder::string(ev.name)});
 
-    // Create Zeek enum types for exported Spicy enums. We do this here
-    // mainly for when compiling C+ code offline. When running live inside
-    // Zeek, we also do it earlier through the GlueBuilder itself so that the
-    // new types are already available when scripts are parsed. (And
-    // registering twice isn't a problem.)
-    for ( auto&& e : _driver->publicEnumTypes() ) {
-        auto labels = hilti::rt::transform(e.type.as<type::Enum>().labels(), [](const auto& l) {
-            return builder::tuple({builder::string(l.get().id()), builder::integer(l.get().value())});
-        });
-
-        preinit_body.addCall("zeek_rt::register_enum_type", {builder::string(e.id.namespace_()),
-                                                             builder::string(e.id.local()), builder::vector(labels)});
+    // Create Zeek types for exported Spicy types.
+    for ( const auto& [tinfo, id] : _driver->exportedTypes() ) {
+        if ( auto type = createZeekType(tinfo.type, id) )
+            preinit_body.addCall("zeek_rt::register_type",
+                                 {builder::string(id.namespace_()), builder::string(id.local()), *type});
+        else
+            hilti::logger().error(hilti::util::fmt("cannot export Spicy type '%s': %s", id, type.error()),
+                                  tinfo.location);
     }
 
-
     for ( auto&& [id, m] : _spicy_modules ) {
         // Import runtime module.
         auto import_ = hilti::builder::import(ID("zeek_rt"), ".hlt");
@@ -869,8 +949,8 @@ bool GlueCompiler::compile() {
             m->spicy_module->add(std::move(import_));
         }
 
-        auto unit = hilti::Unit::fromModule(_driver->context(), std::move(*m->spicy_module), ".spicy");
-        _driver->addInput(std::move(unit));
+        auto unit = hilti::Unit::fromModule(_driver->context(), *m->spicy_module, ".spicy");
+        _driver->addInput(unit);
     }
 
     if ( ! preinit_body.empty() ) {
@@ -880,8 +960,8 @@ bool GlueCompiler::compile() {
         init_module.add(std::move(preinit_function));
     }
 
-    auto unit = hilti::Unit::fromModule(_driver->context(), std::move(init_module), ".hlt");
-    _driver->addInput(std::move(unit));
+    auto unit = hilti::Unit::fromModule(_driver->context(), init_module, ".hlt");
+    _driver->addInput(unit);
     return true;
 }
 
@@ -891,11 +971,12 @@ bool GlueCompiler::PopulateEvents() {
             // Already done.
             continue;
 
-        UnitInfo uinfo;
+        TypeInfo uinfo;
 
         // If we find the path itself, it's refering to a unit type directly;
         // then add a "%done" to form the hook name.
-        if ( auto ui = _driver->lookupUnit(ev.path) ) {
+        if ( auto ui = _driver->lookupType<spicy::type::Unit>(ev.path) ) {
+            // TOOD: Check that it's a unit type.
             uinfo = *ui;
             ev.unit = ev.path;
             ev.hook = ev.unit + hilti::ID("0x25_done");
@@ -910,7 +991,7 @@ bool GlueCompiler::PopulateEvents() {
                 return false;
             }
 
-            if ( auto ui = _driver->lookupUnit(ev.unit) ) {
+            if ( auto ui = _driver->lookupType(ev.unit) ) {
                 uinfo = *ui;
                 ev.hook = ev.path;
             }
@@ -1039,13 +1120,16 @@ bool GlueCompiler::CreateSpicyHook(glue::Event* ev) {
 
         std::vector<std::string> fmt_ctrls(fmt_args.size() - 1, "%s");
         auto fmt_str = hilti::util::fmt("-> event %%s(%s)", hilti::util::join(fmt_ctrls, ", "));
-        auto msg = builder::modulo(builder::string(fmt_str), builder::tuple(std::move(fmt_args)));
+        auto msg = builder::modulo(builder::string(fmt_str), builder::tuple(fmt_args));
         auto call = builder::call("zeek_rt::debug", {std::move(msg)});
         body.addExpression(call);
     }
 
+    // Store reference to handler locally to avoid repeated lookups through globals store.
+    body.addLocal("handler", builder::id(handler_id), meta);
+
     // Nothing to do if there's not handler defined.
-    auto have_handler = builder::call("zeek_rt::have_handler", {builder::id(handler_id)}, meta);
+    auto have_handler = builder::call("zeek_rt::have_handler", {builder::id("handler")}, meta);
     auto exit_ = body.addIf(builder::not_(have_handler), meta);
     exit_->addReturn(meta);
 
@@ -1077,7 +1161,7 @@ bool GlueCompiler::CreateSpicyHook(glue::Event* ev) {
             }
 
             auto ztype = builder::call("zeek_rt::event_arg_type",
-                                       {builder::id(handler_id), builder::integer(i), location(e)}, meta);
+                                       {builder::id("handler"), builder::integer(i), location(e)}, meta);
             val = builder::call("zeek_rt::to_val", {std::move(*expr), ztype, location(e)}, meta);
         }
 
@@ -1085,17 +1169,196 @@ bool GlueCompiler::CreateSpicyHook(glue::Event* ev) {
         i++;
     }
 
-    body.addCall("zeek_rt::raise_event", {builder::id(handler_id), builder::move(builder::id("args")), location(*ev)},
+    body.addCall("zeek_rt::raise_event", {builder::id("handler"), builder::move(builder::id("args")), location(*ev)},
                  meta);
 
     auto attrs = hilti::AttributeSet({hilti::Attribute("&priority", builder::integer(ev->priority))});
-    auto unit_hook = spicy::Hook({}, body.block(), spicy::Engine::All, {}, meta);
-    auto hook_decl = spicy::declaration::UnitHook(ev->hook, std::move(unit_hook), meta);
+    auto unit_hook = spicy::Hook(ev->parameters, body.block(), spicy::Engine::All, {}, meta);
+    auto hook_decl = spicy::declaration::UnitHook(ev->hook, unit_hook, meta);
     ev->spicy_module->spicy_module->add(Declaration(hook_decl));
 
     return true;
 }
 
+
 hilti::Expression GlueCompiler::location(const glue::Event& ev) { return builder::string(ev.location); }
 
 hilti::Expression GlueCompiler::location(const glue::ExpressionAccessor& e) { return builder::string(e.location); }
+
+namespace {
+// Visitor creasting code to instantiate a Zeek type corresponding to a give
+// HILTI type.
+//
+// Note: Any logic changes here must be reflected in the plugin driver's
+// corresponding `VisitorZeekType` as well.
+struct VisitorZeekType : hilti::visitor::PreOrder<hilti::Result<hilti::Expression>, VisitorZeekType> {
+    VisitorZeekType(const GlueCompiler* gc) : gc(gc) {}
+
+    const GlueCompiler* gc;
+
+    std::set<hilti::ID> zeek_types;
+    std::optional<hilti::ID> id;
+
+    result_t base_type(const char* tag) { return builder::call("zeek_rt::create_base_type", {builder::id(tag)}); }
+
+    result_t createZeekType(const hilti::Type& t, const std::optional<hilti::ID>& id_ = {}) {
+        if ( id_ )
+            id = id_;
+        else if ( auto x = t.typeID() )
+            id = *x;
+        else
+            id = std::nullopt;
+
+        if ( id ) {
+            // Avoid infinite recursion.
+            if ( zeek_types.count(*id) )
+                return hilti::result::Error("type is self-recursive");
+
+            zeek_types.insert(*id);
+        }
+
+        auto x = dispatch(t);
+        if ( ! x )
+            return hilti::result::Error(
+                hilti::util::fmt("no support for automatic conversion into a Zeek type (%s)", t.typename_()));
+
+        zeek_types.erase(*id);
+        return *x;
+    }
+
+    result_t operator()(const hilti::type::Address& t) { return base_type("zeek_rt::ZeekTypeTag::Addr"); }
+    result_t operator()(const hilti::type::Bool& t) { return base_type("zeek_rt::ZeekTypeTag::Bool"); }
+    result_t operator()(const hilti::type::Bytes& t) { return base_type("zeek_rt::ZeekTypeTag::String"); }
+    result_t operator()(const hilti::type::Interval& t) { return base_type("zeek_rt::ZeekTypeTag::Interval"); }
+    result_t operator()(const hilti::type::Port& t) { return base_type("zeek_rt::ZeekTypeTag::Port"); }
+    result_t operator()(const hilti::type::Real& t) { return base_type("zeek_rt::ZeekTypeTag::Double"); }
+    result_t operator()(const hilti::type::SignedInteger& t) { return base_type("zeek_rt::ZeekTypeTag::Int"); }
+    result_t operator()(const hilti::type::String& t) { return base_type("zeek_rt::ZeekTypeTag::String"); }
+    result_t operator()(const hilti::type::Time& t) { return base_type("zeek_rt::ZeekTypeTag::Time"); }
+    result_t operator()(const hilti::type::UnsignedInteger& t) { return base_type("zeek_rt::ZeekTypeTag::Count"); }
+
+    result_t operator()(const hilti::type::Enum& t) {
+        assert(id);
+
+        auto labels = hilti::rt::transform(t.labels(), [](const auto& l) {
+            return builder::tuple({builder::string(l.get().id()), builder::integer(l.get().value())});
+        });
+
+        return builder::call("zeek_rt::create_enum_type", {builder::string(id->namespace_()),
+                                                           builder::string(id->local()), builder::vector(labels)});
+    }
+
+    result_t operator()(const hilti::type::Map& t) {
+        auto key = createZeekType(t.keyType());
+        if ( ! key )
+            return key.error();
+
+        auto value = createZeekType(t.valueType());
+        if ( ! value )
+            return value.error();
+
+        return builder::call("zeek_rt::create_table_type", {*key, *value});
+    }
+
+    result_t operator()(const hilti::type::Optional& t) { return createZeekType(t.dereferencedType()); }
+
+    result_t operator()(const hilti::type::Set& t) {
+        auto elem = createZeekType(t.elementType());
+        if ( ! elem )
+            return elem.error();
+
+        return builder::call("zeek_rt::create_table_type", {*elem, builder::null()});
+    }
+
+    result_t operator()(const hilti::type::Struct& t) {
+        assert(id);
+
+        std::vector<hilti::Expression> fields;
+        for ( const auto& f : t.fields() ) {
+            auto ztype = createZeekType(f.type());
+            if ( ! ztype )
+                return ztype.error();
+
+            fields.emplace_back(builder::tuple({builder::string(f.id()), *ztype, builder::bool_(f.isOptional())}));
+        }
+
+        return builder::call("zeek_rt::create_record_type", {builder::string(id->namespace_()),
+                                                             builder::string(id->local()), builder::vector(fields)});
+    }
+
+    result_t operator()(const spicy::type::Tuple& t) {
+        std::vector<hilti::Expression> fields;
+        for ( const auto& f : t.elements() ) {
+            if ( ! f.id() )
+                return hilti::result::Error("can only convert tuple types with all-named fields to Zeek");
+
+            auto ztype = createZeekType(f.type());
+            if ( ! ztype )
+                return ztype.error();
+
+            fields.emplace_back(builder::tuple({builder::string(*f.id()), *ztype, builder::bool_(false)}));
+        }
+
+        return builder::call("zeek_rt::create_record_type", {builder::string(id->namespace_()),
+                                                             builder::string(id->local()), builder::vector(fields)});
+    }
+
+    result_t operator()(const spicy::type::Unit& t) {
+        assert(id);
+
+        std::vector<hilti::Expression> fields;
+        for ( const auto& f : gc->recordFields(t) ) {
+            auto ztype = createZeekType(std::get<1>(f));
+            if ( ! ztype )
+                return ztype.error();
+
+            fields.emplace_back(
+                builder::tuple({builder::string(std::get<0>(f)), *ztype, builder::bool_(std::get<2>(f))}));
+        }
+
+        return builder::call("zeek_rt::create_record_type", {builder::string(id->namespace_()),
+                                                             builder::string(id->local()), builder::vector(fields)});
+    }
+
+    result_t operator()(const hilti::type::Vector& t) {
+        auto elem = createZeekType(t.elementType());
+        if ( ! elem )
+            return elem.error();
+
+        return builder::call("zeek_rt::create_vector_type", {*elem});
+    }
+};
+} // namespace
+
+hilti::Result<hilti::Expression> GlueCompiler::createZeekType(const hilti::Type& t, const hilti::ID& id) const {
+    return VisitorZeekType(this).createZeekType(t, id);
+}
+
+namespace {
+struct VisitorUnitFields : hilti::visitor::PreOrder<void, VisitorUnitFields> {
+    // NOTE: Align this logic with struct generation in Spicy's unit builder.
+    std::vector<GlueCompiler::RecordField> fields;
+
+    void operator()(const spicy::type::unit::item::Field& f, position_t p) {
+        if ( f.isTransient() || f.parseType().isA<hilti::type::Void>() )
+            return;
+
+        fields.emplace_back(f.id(), f.itemType(), true);
+    }
+
+    void operator()(const spicy::type::unit::item::Variable& f, const position_t p) {
+        fields.emplace_back(f.id(), f.itemType(), f.isOptional());
+    }
+
+    // TODO: void operator()(const spicy::type::unit::item::Switch & f, const position_t p) {
+};
+} // namespace
+
+std::vector<GlueCompiler::RecordField> GlueCompiler::recordFields(const ::spicy::type::Unit& unit) {
+    VisitorUnitFields unit_field_converter;
+
+    for ( const auto& i : unit.items() )
+        unit_field_converter.dispatch(i);
+
+    return std::move(unit_field_converter.fields);
+}
