@@ -12,7 +12,7 @@
 #include <hilti/rt/iterator.h>
 #include <hilti/rt/json-fwd.h>
 #include <hilti/rt/result.h>
-#include <hilti/rt/types/integer.h>
+#include <hilti/rt/safe-int.h>
 #include <hilti/rt/types/string.h>
 #include <hilti/rt/types/time.h>
 #include <hilti/rt/types/vector.h>
@@ -30,14 +30,17 @@ class View;
 namespace bytes {
 
 /** For Bytes::Strip, which side to strip from. */
-enum class Side : int64_t {
-    Left,  /**< left side */
-    Right, /**< right side */
-    Both   /**< left and right sides */
-};
+HILTI_RT_ENUM_WITH_DEFAULT(Side, Left,
+                           Left,  // left side
+                           Right, // right side
+                           Both   // left and right side
+);
 
-/** For Bytes::Decode, which character set to use. */
-enum class Charset : int64_t { Undef, UTF8, ASCII };
+/** For bytes decoding, which character set to use. */
+HILTI_RT_ENUM(Charset, Undef, UTF8, ASCII);
+
+/** For bytes decoding, how to handle decoding errors. */
+using DecodeErrorStrategy = string::DecodeErrorStrategy;
 
 class Iterator {
     using B = std::string;
@@ -164,14 +167,15 @@ public:
     using Base::data;
 
     /**
-     * Creates a bytes instance from a UTF8 string, transforming the contents
-     * into a binary representation defined by a specified character set.
+     * Creates a bytes instance from a raw string representation
+     * encoded in a specified character set.
      *
-     * @param s string assumed to be in UTF8 (as all runtime strings)
-     * @param cs character set to use for the binary encoding
+     * @param s raw data
+     * @param cs character set the raw data is assumed to be encoded in
+     * @param errors how to handle errors when decoding the data
      * @return bytes instances encoding *s* in character set *cs*
      */
-    Bytes(std::string s, bytes::Charset cs);
+    Bytes(std::string s, bytes::Charset cs, bytes::DecodeErrorStrategy errors = bytes::DecodeErrorStrategy::REPLACE);
 
     Bytes(Base&& str) : Base(std::move(str)), _control(std::make_shared<Base*>(static_cast<Base*>(this))) {}
     Bytes(const Bytes& xs) : Base(xs), _control(std::make_shared<Base*>(static_cast<Base*>(this))) {}
@@ -297,17 +301,16 @@ public:
     /**
      * Extracts a fixed number of bytes from the data
      *
-     * @tparam N number of bytes to extract
      * @param dst array to writes bytes into
+     * @param n number of bytes to extract
      * @return new bytes instance that has the first *N* bytes removed.
      */
-    template<int N>
-    Bytes extract(unsigned char (&dst)[N]) const {
-        if ( N > size() )
+    Bytes extract(unsigned char* dst, uint64_t n) const {
+        if ( n > size() )
             throw InvalidArgument("insufficient data in source");
 
-        memcpy(dst, data(), N);
-        return sub(N, std::string::npos);
+        memcpy(dst, data(), n);
+        return sub(n, std::string::npos);
     }
 
     /**
@@ -315,9 +318,11 @@ public:
      * specified character set.
      *
      * @param cs character set to assume the binary data to be encoded in
+     * @param errors how to handle errors when decoding the data
      * @return UTF8 string
      */
-    std::string decode(bytes::Charset cs) const;
+    std::string decode(bytes::Charset cs,
+                       bytes::DecodeErrorStrategy errors = bytes::DecodeErrorStrategy::REPLACE) const;
 
     /** Returns true if the data begins with a given, other bytes instance. */
     bool startsWith(const Bytes& b) const { return hilti::rt::startsWith(*this, b); }
@@ -328,16 +333,23 @@ public:
      * back afterwards.
      *
      * @param cs character set for decoding/encoding
+     * @param errors how to handle errors when decoding/encoding the data
      * @return an upper case version of the instance
      */
-    Bytes upper(bytes::Charset cs) const { return Bytes(hilti::rt::string::upper(decode(cs)), cs); }
+    Bytes upper(bytes::Charset cs, bytes::DecodeErrorStrategy errors = bytes::DecodeErrorStrategy::REPLACE) const {
+        return Bytes(hilti::rt::string::upper(decode(cs, errors), errors), cs, errors);
+    }
 
-    /** Returns an upper-case version of the instance.
+    /**
+     * Returns an upper-case version of the instance.
      *
      * @param cs character set for decoding/encoding
+     * @param errors how to handle errors when decoding/encoding the data
      * @return a lower case version of the instance
      */
-    Bytes lower(bytes::Charset cs) const { return Bytes(hilti::rt::string::lower(decode(cs)), cs); }
+    Bytes lower(bytes::Charset cs, bytes::DecodeErrorStrategy errors = bytes::DecodeErrorStrategy::REPLACE) const {
+        return Bytes(hilti::rt::string::lower(decode(cs, errors), errors), cs, errors);
+    }
 
     /**
      * Removes leading and/or trailing sequences of all characters of a set
@@ -537,6 +549,7 @@ namespace detail::adl {
 std::string to_string(const Bytes& x, adl::tag /*unused*/);
 std::string to_string(const bytes::Side& x, adl::tag /*unused*/);
 std::string to_string(const bytes::Charset& x, adl::tag /*unused*/);
+std::string to_string(const bytes::DecodeErrorStrategy& x, adl::tag /*unused*/);
 } // namespace detail::adl
 
 } // namespace hilti::rt
