@@ -7,6 +7,8 @@
 #include <cinttypes>
 #include <clocale>
 #include <cstring>
+#include <memory>
+#include <vector>
 
 #include <hilti/rt/configuration.h>
 #include <hilti/rt/context.h>
@@ -27,12 +29,14 @@ void hilti::rt::init() {
     if ( ! globalState()->configuration )
         globalState()->configuration = std::make_unique<hilti::rt::Configuration>();
 
-    if ( auto debug_out = globalState()->configuration->debug_out )
-        globalState()->debug_logger = std::make_unique<hilti::rt::detail::DebugLogger>(*debug_out);
-    else
-        globalState()->debug_logger = std::make_unique<hilti::rt::detail::DebugLogger>("/dev/stderr");
+    if ( ! globalState()->configuration->debug_streams.empty() ) {
+        if ( auto debug_out = globalState()->configuration->debug_out )
+            globalState()->debug_logger = std::make_unique<hilti::rt::detail::DebugLogger>(*debug_out);
+        else
+            globalState()->debug_logger = std::make_unique<hilti::rt::detail::DebugLogger>("/dev/stderr");
 
-    globalState()->debug_logger->enable(globalState()->configuration->debug_streams);
+        globalState()->debug_logger->enable(globalState()->configuration->debug_streams);
+    }
 
     HILTI_RT_DEBUG("libhilti", "initializing runtime");
 
@@ -94,3 +98,22 @@ void hilti::rt::detail::registerModule(HiltiModule module) {
 
     globalState()->hilti_modules.emplace_back(module);
 }
+
+static std::unique_ptr<std::vector<void (*)()>> _registered_preinit_functions;
+
+RegisterManualPreInit::RegisterManualPreInit(void (*f)()) {
+    if ( ! _registered_preinit_functions )
+        _registered_preinit_functions = std::make_unique<std::vector<void (*)()>>();
+
+    _registered_preinit_functions->emplace_back(f);
+}
+
+void hilti::rt::executeManualPreInits() {
+    if ( ! _registered_preinit_functions )
+        return;
+
+    for ( const auto& f : *_registered_preinit_functions )
+        (*f)();
+
+    _registered_preinit_functions.reset();
+}
