@@ -35,9 +35,12 @@ inline const DebugStream Compiler("compiler");
 inline const DebugStream Driver("driver");
 } // namespace hilti::logging::debug
 
+constexpr int OPT_CXX_LINK = 1000;
+
 static struct option long_driver_options[] = {{"abort-on-exceptions", required_argument, nullptr, 'A'},
                                               {"show-backtraces", required_argument, nullptr, 'B'},
                                               {"compiler-debug", required_argument, nullptr, 'D'},
+                                              {"cxx-link", required_argument, nullptr, OPT_CXX_LINK},
                                               {"debug", no_argument, nullptr, 'd'},
                                               {"debug-addl", required_argument, nullptr, 'X'},
                                               {"disable-optimizations", no_argument, nullptr, 'g'},
@@ -45,9 +48,9 @@ static struct option long_driver_options[] = {{"abort-on-exceptions", required_a
                                               {"help", no_argument, nullptr, 'h'},
                                               {"keep-tmps", no_argument, nullptr, 'T'},
                                               {"library-path", required_argument, nullptr, 'L'},
-                                              {"optimize", no_argument, nullptr, 'O'},
                                               {"output", required_argument, nullptr, 'o'},
                                               {"output-c++", no_argument, nullptr, 'c'},
+                                              {"output-c++-files", no_argument, nullptr, 'x'},
                                               {"output-hilti", no_argument, nullptr, 'p'},
                                               {"execute-code", no_argument, nullptr, 'j'},
                                               {"output-linker", no_argument, nullptr, 'l'},
@@ -109,6 +112,7 @@ void Driver::usage() {
            "  -o | --output-to <path>         Path for saving output.\n"
            "  -p | --output-hilti             Just output parsed HILTI code again.\n"
            "  -v | --version                  Print version information.\n"
+           "  -x | --output-c++ <prefix>      Output generated C++ code into set of files.\n"
            "  -A | --abort-on-exceptions      When executing compiled code, abort() instead of throwing HILTI "
            "exceptions.\n"
            "  -B | --show-backtraces          Include backtraces when reporting unhandled exceptions.\n"
@@ -118,7 +122,6 @@ void Driver::usage() {
            "  -E | --output-code-dependencies Output list of dependencies for all compiled modules that require "
            "separate compilation of their own.\n"
            "  -L | --library-path <path>      Add path to list of directories to search when importing modules.\n"
-           "  -O | --optimize                 Build optimized release version of generated code.\n"
            "  -P | --output-prototypes        Output C++ header with prototypes for public functionality.\n"
            "  -R | --report-times             Report a break-down of compiler's execution time.\n"
            "  -S | --skip-dependencies        Do not automatically compile dependencies during JIT.\n"
@@ -126,6 +129,8 @@ void Driver::usage() {
            "  -V | --skip-validation          Don't validate ASTs (for debugging only).\n"
            "  -X | --debug-addl <addl>        Implies -d and adds selected additional instrumentation "
            "(comma-separated; see 'help' for list).\n"
+           "       --cxx-link <lib>           Link specified static archive or shared library during JIT or to "
+           "produced HLTO file. Can be given multiple times.\n"
         << addl_usage
         << "\n"
            "Inputs can be "
@@ -239,7 +244,7 @@ void Driver::dumpUnit(const Unit& unit) {
     }
 
     if ( auto cxx = unit.cxxCode() ) {
-        ID id = (unit.isCompiledHILTI() ? unit.uniqueID(): ID(unit.cxxCode()->id()));
+        ID id = (unit.isCompiledHILTI() ? unit.uniqueID() : ID(unit.cxxCode()->id()));
         auto output_path = util::fmt("dbg.%s.cc", id);
         if ( auto out = openOutput(util::fmt("dbg.%s.cc", id)) ) {
             HILTI_DEBUG(logging::debug::Driver, fmt("saving C++ code for module %s to %s", id, output_path));
@@ -252,7 +257,7 @@ Result<Nothing> Driver::parseOptions(int argc, char** argv) {
     int num_output_types = 0;
 
     opterr = 0; // don't print errors
-    std::string option_string = "ABlL:OcCpPvjhvVdX:o:D:TUEeSRg" + hookAddCommandLineOptions();
+    std::string option_string = "ABlL:cCpPvjhvx:VdX:o:D:TUEeSRg" + hookAddCommandLineOptions();
 
     while ( true ) {
         int c = getopt_long(argc, argv, option_string.c_str(), long_driver_options, nullptr);
@@ -270,6 +275,19 @@ Result<Nothing> Driver::parseOptions(int argc, char** argv) {
                 ++num_output_types;
                 break;
 
+            case 'x':
+                _driver_options.output_cxx = true;
+                _driver_options.output_cxx_prefix = optarg;
+                _driver_options.execute_code = false;
+                _driver_options.include_linker = true;
+                _compiler_options.cxx_namespace_extern =
+                    hilti::util::fmt("hlt_%s", hilti::rt::filesystem::path(optarg).stem().string());
+                _compiler_options.cxx_namespace_intern =
+                    hilti::util::fmt("__hlt_%s", hilti::rt::filesystem::path(optarg).stem().string());
+
+                ++num_output_types;
+                break;
+
             case 'C': {
                 _driver_options.dump_code = true;
                 break;
@@ -348,12 +366,10 @@ Result<Nothing> Driver::parseOptions(int argc, char** argv) {
                 ++num_output_types;
                 break;
 
-            case 'L': _compiler_options.library_paths.emplace_back(std::string(optarg)); break;
+            case 'L': _compiler_options.library_paths.emplace_back(optarg); break;
 
             case 'o': _driver_options.output_path = std::string(optarg); break;
 
-            case 'O': _compiler_options.optimize = true; break;
-
             case 'p':
                 _driver_options.output_hilti = true;
                 ++num_output_types;
@@ -381,6 +397,8 @@ Result<Nothing> Driver::parseOptions(int argc, char** argv) {
 
             case 'V': _compiler_options.skip_validation = true; break;
 
+            case OPT_CXX_LINK: _compiler_options.cxx_link.emplace_back(optarg); break;
+
             case 'h': usage(); return Nothing();
 
             case '?': usage(); return error("unknown option");
@@ -408,7 +426,7 @@ Result<Nothing> Driver::parseOptions(int argc, char** argv) {
 
     if ( ! _compiler_options.debug ) {
         if ( _compiler_options.debug_trace || _compiler_options.debug_flow )
-            return error("cannot use --optimize with --cgdebug");
+            return error("must use --debug with --cgdebug");
     }
 
     if ( _driver_options.execute_code and ! _driver_options.output_path.empty() ) {
@@ -691,7 +709,8 @@ Result<Nothing> Driver::_resolveUnitsWithPlugin(const Plugin& plugin, std::vecto
             logging::DebugPushIndent _(logging::debug::Compiler);
             HILTI_DEBUG(logging::debug::Compiler,
                         fmt("dependencies: %s",
-                            util::join(util::transform(u->dependencies(), [](const auto& u) { return u.lock()->uniqueID(); }),
+                            util::join(util::transform(u->dependencies(),
+                                                       [](const auto& u) { return u.lock()->uniqueID(); }),
                                        ", ")));
         }
 
@@ -1060,7 +1079,8 @@ Result<Nothing> Driver::outputUnits() {
                 if ( ! output )
                     return output.error();
 
-                HILTI_DEBUG(logging::debug::Driver, fmt("saving C++ code for module %s to %s", unit->uniqueID(), cxx_path));
+                HILTI_DEBUG(logging::debug::Driver,
+                            fmt("saving C++ code for module %s to %s", unit->uniqueID(), cxx_path));
                 cxx->save(*output);
             }
 
@@ -1077,7 +1097,8 @@ Result<Nothing> Driver::outputUnits() {
             if ( _driver_options.output_dependencies != driver::Dependencies::None ) {
                 const bool code_only = (_driver_options.output_dependencies == driver::Dependencies::Code);
 
-                for ( const auto& unit_ : context()->lookupDependenciesForUnit(unit->cacheIndex(), unit->extension()) ) {
+                for ( const auto& unit_ :
+                      context()->lookupDependenciesForUnit(unit->cacheIndex(), unit->extension()) ) {
                     auto unit = unit_.lock();
                     if ( code_only && ! unit->requiresCompilation() )
                         continue;
@@ -1260,7 +1281,8 @@ void Driver::_dumpDeclarations(const std::shared_ptr<Unit>& unit, const Plugin&
         return;
 
     logger().debugSetIndent(logging::debug::AstDeclarations, 0);
-    HILTI_DEBUG(logging::debug::AstDeclarations, fmt("# [%s] %s", pluginForUnit(unit).get().component, unit->uniqueID()));
+    HILTI_DEBUG(logging::debug::AstDeclarations,
+                fmt("# [%s] %s", pluginForUnit(unit).get().component, unit->uniqueID()));
 
     for ( const auto i : visitor::PreOrder<>().walk(unit->module()) ) {
         auto decl = i.node.tryAs<Declaration>();
@@ -1278,7 +1300,7 @@ void Driver::_saveIterationAST(const std::shared_ptr<Unit>& unit, const Plugin&
     if ( ! logger().isEnabled(logging::debug::AstDumpIterations) )
         return;
 
-    std::ofstream out(fmt("ast-%s-%d.tmp", plugin.component, round));
+    std::ofstream out(fmt("ast-%s-%s-%d.tmp", plugin.component, unit->id(), round));
     _dumpAST(unit, out, plugin, prefix, round);
 }
 
