@@ -21,19 +21,8 @@
 #include <hilti/compiler/detail/cxx/unit.h>
 #include <hilti/compiler/jit.h>
 
-// GCC-13 warns about code in reproc++. This is fixed upstream with
-// DaanDeMeyer/reproc@0b23d88894ccedde04537fa23ea55cb2f8365342, but that patch
-// has not landed in a release yet. Disable the warning if the compiler knows
-// about it.
-//
-// TODO(bbannier): Drop this once reproc puts out a release officially supporting gcc-13.
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wpragmas"
-#pragma GCC diagnostic ignored "-Wunknown-warning-option"
-#pragma GCC diagnostic ignored "-Wchanges-meaning"
 #include <reproc++/drain.hpp>
 #include <reproc++/reproc.hpp>
-#pragma GCC diagnostic pop
 
 using namespace hilti;
 
@@ -180,8 +169,7 @@ bool CxxCode::save(std::ostream& out) const {
 JIT::JIT(const std::shared_ptr<Context>& context, bool dump_code)
     : _context(context),
       _dump_code(dump_code),
-      _hash(rt::hashCombine(hilti::configuration().hash(),
-                            std::hash<std::string>{}(hilti::rt::filesystem::current_path().string()))) {}
+      _hash(std::hash<std::string>{}(hilti::rt::filesystem::current_path().string())) {}
 
 JIT::~JIT() { _finish(); }
 
@@ -308,9 +296,8 @@ hilti::Result<Nothing> JIT::_compile() {
 
         if ( auto path = getenv("HILTI_CXX_INCLUDE_DIRS") ) {
             for ( auto&& dir : hilti::rt::split(path, ":") ) {
-                if ( dir.size() ) {
-                    args.emplace_back("-I");
-                    args.emplace_back(dir);
+                if ( ! dir.empty() ) {
+                    args.insert(args.begin(), {"-I", std::string(dir)});
                 }
             }
         }
@@ -396,6 +383,12 @@ hilti::Result<std::shared_ptr<const Library>> JIT::_link() {
         }
     }
 
+    // Add additional shared libraries or static archives to the link. This needs to happen
+    // after we added the objects to make sure we pull in symbols used in the objects.
+    for ( const auto& lib : options().cxx_link )
+        if ( ! lib.empty() )
+            args.emplace_back(lib);
+
     // We are using the compiler as a linker here, no need to use a compiler launcher.
     // Since we are writing to a random temporary file non of this would cache anyway.
     if ( auto rc = _runner._scheduleJob(hilti::configuration().cxx, std::move(args)); ! rc )
@@ -446,7 +439,7 @@ Result<JIT::JobRunner::JobID> JIT::JobRunner::_scheduleJob(const hilti::rt::file
     auto jid = ++_job_counter;
     HILTI_DEBUG(logging::debug::Jit, util::fmt("[job %u] %s", jid, util::join(cmdline, " ")));
 
-    _jobs_pending.emplace_back(std::make_tuple(jid, cmdline));
+    _jobs_pending.emplace_back(jid, cmdline);
     return jid;
 }
 
