@@ -234,7 +234,10 @@ struct Visitor : public visitor::PostOrder<void, Visitor> {
             field_types.emplace_back(declaration::Field(f.id(), f.expression().type(), std::nullopt, f.id().meta()));
         }
 
-        auto ntype = type::Struct(std::move(field_types), u.meta());
+        Type ntype = type::Struct(type::Struct::AnonymousStruct{}, std::move(field_types), u.meta());
+        if ( auto id = u.type().typeID() )
+            ntype.setTypeID(*id);
+
         logChange(p.node, ntype);
         p.node.as<ctor::Struct>().setType(ntype);
         modified = true;
@@ -272,7 +275,7 @@ struct Visitor : public visitor::PostOrder<void, Visitor> {
     }
 
     void operator()(const declaration::Function& u, position_t p) {
-        if ( u.linkage() != declaration::Linkage::Struct ) {
+        if ( u.linkage() != declaration::Linkage::Struct && u.id().namespace_() ) {
             // See if the namespace refers to a struct. If so, change linkage
             // because that's what the normalizer will look for when linking
             // methods to their parent type.
@@ -342,6 +345,18 @@ struct Visitor : public visitor::PostOrder<void, Visitor> {
         }
     }
 
+    void operator()(const expression::Keyword& e, position_t p) {
+        if ( e.kind() != expression::keyword::Kind::Scope )
+            return;
+
+        if ( type::isResolved(e.type()) )
+            return;
+
+        logChange(p.node, hilti::type::String());
+        p.node.as<expression::Keyword>().setType(hilti::type::String());
+        modified = true;
+    }
+
     void operator()(const expression::ListComprehension& e, position_t p) {
         if ( ! type::isResolved(e.type()) && type::isResolved(e.output().type()) ) {
             logChange(p.node, e.output().type());
@@ -390,7 +405,7 @@ struct Visitor : public visitor::PostOrder<void, Visitor> {
                 return;
 
             auto n = expression::ResolvedID(resolved->second, NodeRef(resolved->first), u.meta());
-            if ( ! type::isResolved(n.type()) )
+            if ( ! expression::isResolved(n) )
                 return;
 
             logChange(p.node, n);
@@ -573,7 +588,7 @@ bool Visitor::resolveOperator(const expression::UnresolvedOperator& u, position_
     modified = true;
 
 #ifndef NDEBUG
-    const Expression& new_op = p.node.as<expression::ResolvedOperator>();
+    const auto& new_op = p.node.as<Expression>();
     HILTI_DEBUG(logging::debug::Operator, util::fmt("=> resolved to %s (result: %s, expression is %s)", p.node.render(),
                                                     new_op, (new_op.isConstant() ? "const" : "non-const")));
 #endif
@@ -755,7 +770,8 @@ bool Visitor::resolveMethodCall(const expression::UnresolvedOperator& u, positio
 
     auto fields = stype->fields(member->id());
     if ( fields.empty() ) {
-        p.node.addError(util::fmt("struct type does not have a method `%s`", member->id()));
+        p.node.addError(
+            util::fmt("struct type %s does not have a method `%s`", stype->meta().location(), member->id()));
         return false; // Continue trying to find another match.
     }
 
@@ -908,6 +924,11 @@ std::vector<Node> Visitor::matchOverloads(const std::vector<Operator>& candidate
 
         auto r = candidate.instantiate(nops->second, meta);
 
+        // Fold any constants right here in case downstream resolving depends
+        // on finding a constant (like for coercion).
+        if ( auto ctor = detail::foldConstant(r); ctor && *ctor )
+            r = expression::Ctor(**ctor, r.meta());
+
         // Some operators may not be able to determine their type before the
         // resolver had a chance to provide the information needed. They will
         // return "auto" in that case (specifically, that's the case for Spicy
