@@ -18,6 +18,7 @@
 #include <hilti/rt/types/port.h>
 #include <hilti/rt/types/reference.h>
 #include <hilti/rt/types/struct.h>
+#include <hilti/rt/util.h>
 
 #include <spicy/rt/filter.h>
 #include <spicy/rt/global-state.h>
@@ -29,14 +30,14 @@
 namespace spicy::rt {
 
 /** Defines the direction a `ParserPort` applies to. */
-enum class Direction { Originator, Responder, Both, Undef };
+HILTI_RT_ENUM(Direction, Originator, Responder, Both, Undef);
 
 } // namespace spicy::rt
 
 namespace hilti::rt::detail::adl {
 
 inline std::string to_string(const ::spicy::rt::Direction& x, adl::tag /*unused*/) {
-    switch ( x ) {
+    switch ( x.value() ) {
         case spicy::rt::Direction::Originator: return "originator";
         case spicy::rt::Direction::Responder: return "responder";
         case spicy::rt::Direction::Both: return "both";
@@ -315,6 +316,20 @@ public:
     ~MissingData() override; /* required to create vtable, see hilti::rt::Exception */
 };
 
+/**
+ * Reports a confirmation to the host application indicating that the parser
+ * appears to be processing the expected input format.
+ */
+extern void accept_input();
+
+/**
+ * Reports a violation to the host application indicating that the parser
+ * appears to not be processing the expected input format.
+ *
+ * @param reason user-presentable description of the violation
+ */
+extern void decline_input(const std::string& reason);
+
 namespace detail {
 
 /**
