@@ -68,12 +68,15 @@ hilti::Result<hilti::Nothing> isParseableType(const Type& pt, const type::unit::
                 size_attrs_present.emplace_back(i->tag());
         }
 
-        for ( const auto* attrs_present : {&start_attrs_present, &end_attrs_present, &size_attrs_present} ) {
+        for ( const auto* attrs_present : {&start_attrs_present, &size_attrs_present} ) {
             if ( attrs_present->size() > 1 )
                 return hilti::result::Error(
                     fmt("attributes cannot be combined: %s", hilti::util::join(*attrs_present, ", ")));
         }
 
+        if ( until_attr && until_including_attr )
+            return hilti::result::Error(fmt("attributes cannot be combined: &until, &until-including"));
+
         if ( ! size_attr && start_attrs_present.empty() && end_attrs_present.empty() )
             return hilti::result::Error(
                 "bytes field requires one of &eod, &parse_at, &parse_from, &size, &until, &until-including");
@@ -750,6 +753,14 @@ struct VisitorPost : public hilti::visitor::PreOrder<void, VisitorPost>, public
     }
 
     void operator()(const spicy::type::unit::item::Variable& v, position_t p) {
+        if ( auto attrs = v.attributes() ) {
+            for ( const auto& attr : attrs->attributes() ) {
+                const auto& tag = attr.tag();
+                if ( tag != "&optional" )
+                    error(fmt("attribute '%s' not supported for unit variables", tag), p);
+            }
+        }
+
         if ( v.itemType().isA<type::Sink>() )
             error(
                 "cannot use type 'sink' for unit variables; use either a 'sink' item or a reference to a sink "
@@ -787,8 +798,8 @@ struct VisitorPost : public hilti::visitor::PreOrder<void, VisitorPost>, public
         else if ( hilti::util::startsWith(id, "0x25_") ) {
             auto id_readable = hilti::util::replace(hook.id().local().str(), "0x25_", "%");
 
-            if ( id == "0x25_init" || id == "0x25_done" || id == "0x25_print" ||
-                 id == "0x25_finally" || id == "0x25_rejected" || id == "0x25_confirmed" || id == "0x25_synced" ) {
+            if ( id == "0x25_init" || id == "0x25_done" || id == "0x25_print" || id == "0x25_finally" ||
+                 id == "0x25_rejected" || id == "0x25_confirmed" || id == "0x25_synced" ) {
                 if ( params.size() != 0 )
                     error(fmt("hook '%s' does not take any parameters", id_readable), p, location);
             }
@@ -860,6 +871,16 @@ struct VisitorPost : public hilti::visitor::PreOrder<void, VisitorPost>, public
             error("unit type cannot be a filter, %filter missing", p);
     }
 
+    void operator()(const operator_::unit::ContextConst& n, position_t p) {
+        if ( auto x = n.op0().type().tryAs<type::Unit>(); x && ! x->contextType() )
+            error("context() used with a unit which did not declare %context", p);
+    }
+
+    void operator()(const operator_::unit::ContextNonConst& n, position_t p) {
+        if ( auto x = n.op0().type().tryAs<type::Unit>(); x && ! x->contextType() )
+            error("context() used with a unit which did not declare %context", p);
+    }
+
     void operator()(const operator_::unit::Forward& n, position_t p) {
         if ( auto x = n.op0().type().tryAs<type::Unit>(); x && ! x->isFilter() )
             error("unit type cannot be a filter, %filter missing", p);
@@ -869,6 +890,21 @@ struct VisitorPost : public hilti::visitor::PreOrder<void, VisitorPost>, public
         if ( auto x = n.op0().type().tryAs<type::Unit>(); x && ! x->isFilter() )
             error("unit type cannot be a filter, %filter missing", p);
     }
+
+    void operator()(const spicy::type::Bitfield& b, position_t p) {
+        const auto width = b.width();
+
+        for ( const auto& bit : b.bits() ) {
+            const auto lower = bit.lower();
+            const auto upper = bit.upper();
+
+            if ( lower > upper )
+                error("lower limit needs to be lower than upper limit", p);
+
+            if ( upper >= width )
+                error("upper limit is beyond the width of the bitfield", p);
+        }
+    }
 };
 
 } // anonymous namespace
