@@ -7,15 +7,12 @@ import os
 import sys
 import traceback
 
-import broker
+from . import brokertypes as bt
+from . import controller
 
 from .config import CONFIG
 
-from .controller import Controller
-
-from .consts import (
-    CONFIG_FILE
-)
+from .consts import CONFIG_FILE
 
 from .events import (
     DeployRequest,
@@ -39,7 +36,7 @@ from .events import (
 from .logs import LOG
 
 from .types import (
-    BrokerEnumType,
+    Enum,
     ClusterRole,
     Configuration,
     Instance,
@@ -49,6 +46,11 @@ from .types import (
     Result
 )
 
+# For unit-testing, a central place to adjust where reads from stdin come from
+# and writes to stdout go to. Fiddling with sys.stdin/sys.stdout directly in the
+# tests can be tricky.
+STDIN = sys.stdin
+STDOUT = sys.stdout
 
 # Broker's basic types aren't JSON-serializable, so patch that up
 # in this json.dumps() wrapper for JSON serialization of any object.
@@ -56,14 +58,26 @@ from .types import (
 # dependencies.
 def json_dumps(obj):
     def default(obj):
+        # Check specific Python types:
         if isinstance(obj, ipaddress.IPv4Address):
             return str(obj)
         if isinstance(obj, ipaddress.IPv6Address):
             return str(obj)
-        if isinstance(obj, broker.Port):
-            return str(obj)
-        if isinstance(obj, BrokerEnumType):
+
+        # Specific zeek-client types (types.py):
+        if isinstance(obj, Enum):
             return obj.to_json_data()
+
+        # Specific brokertypes:
+        if isinstance(obj, bt.Port):
+            return str(obj.number)
+        if isinstance(obj, bt.Timespan):
+            return '{}{}'.format(obj.value, obj.unit.value)
+        # Fallback: assume the type's own Python representation is right.
+        # json.dumps() will complain when that does not work.
+        if isinstance(obj, bt.Type):
+            return obj.to_py()
+
         raise TypeError('cannot serialize {} ({})'.format(type(obj), str(obj)))
 
     indent = 2 if CONFIG.getboolean('client', 'pretty_json') else None
@@ -71,13 +85,16 @@ def json_dumps(obj):
 
 
 def create_controller():
-    controller = Controller(CONFIG.get('controller', 'host'),
-                            CONFIG.getint('controller', 'port'))
-
-    if not controller.connect():
+    try:
+        ctl = controller.Controller()
+    except controller.Error as err:
+        LOG.error(str(err))
         return None
 
-    return controller
+    if not ctl.connect():
+        return None
+
+    return ctl
 
 
 def create_parser():
@@ -190,8 +207,11 @@ def create_parser():
     return parser
 
 
-def cmd_deploy(args):
-    controller = create_controller()
+def cmd_deploy(args, controller=None):
+    # The deploy-config command first stages a configuration and then calls this
+    # function to deploy. We re-use its controller, passed to us.
+    if controller is None:
+        controller = create_controller()
     if controller is None:
         return 1
 
@@ -208,7 +228,7 @@ def cmd_deploy(args):
     }
 
     for broker_data in resp.results:
-        res = Result.from_broker(broker_data)
+        res = Result.from_brokertype(broker_data)
 
         if not res.success:
             retval = 1
@@ -251,11 +271,11 @@ def cmd_deploy(args):
         # buffering in the node -> stem -> supervisor pipeline delays the
         # output.)
         if res.data:
-            node_outputs = NodeOutputs.from_broker(res.data)
+            node_outputs = NodeOutputs.from_brokertype(res.data)
             json_data['results']['nodes'][res.node]['stdout'] = node_outputs.stdout
             json_data['results']['nodes'][res.node]['stderr'] = node_outputs.stderr
 
-    print(json_dumps(json_data))
+    print(json_dumps(json_data), file=STDOUT)
     return retval
 
 
@@ -272,7 +292,7 @@ def cmd_get_config(args):
         LOG.error('no response received: %s', msg)
         return 1
 
-    res = Result.from_broker(resp.result)
+    res = Result.from_brokertype(resp.result)
 
     if not res.success:
         msg = res.error if res.error else 'no reason given'
@@ -283,9 +303,9 @@ def cmd_get_config(args):
         LOG.error('received result did not contain configuration data: %s', resp)
         return 1
 
-    config = Configuration.from_broker(res.data)
+    config = Configuration.from_brokertype(res.data)
 
-    with open(args.filename, 'w') if args.filename and args.filename != '-'  else sys.stdout as hdl:
+    with open(args.filename, 'w') if args.filename and args.filename != '-' else STDOUT as hdl:
         if args.as_json:
             hdl.write(json_dumps(config.to_json_data()) + '\n')
         else:
@@ -318,7 +338,7 @@ def cmd_get_id_value(args):
     # task to Python, for our error reporting it's up to us, and we want be
     # reproducible.
 
-    results = [Result.from_broker(broker_data) for broker_data in resp.results]
+    results = [Result.from_brokertype(broker_data) for broker_data in resp.results]
 
     for res in sorted(results):
         if not res.success:
@@ -330,10 +350,18 @@ def cmd_get_id_value(args):
 
         # Upon success, we should always have res.node filled in. But guard anyway.
         if res.node:
-            # res.data is a string containing JSON rendered by Zeek's to_json()
-            # BiF. Parse it into a data structure to render seamlessly.
+            # res.data should be a string containing JSON rendered by Zeek's
+            # to_json() BiF. Parse it into a data structure to render
+            # seamlessly.
+            if not isinstance(res.data, bt.String):
+                json_data['errors'].append({
+                    'source': res.node,
+                    'error': 'invalid result data type {}'.format(repr(res.data))
+                })
+                continue
+
             try:
-                json_data['results'][res.node] = json.loads(res.data)
+                json_data['results'][res.node] = json.loads(res.data.to_py())
             except json.JSONDecodeError as err:
                 json_data['errors'].append({
                     'source': res.node,
@@ -345,7 +373,7 @@ def cmd_get_id_value(args):
             'error': 'result lacking node: {}'.format(res.data),
         })
 
-    print(json_dumps(json_data))
+    print(json_dumps(json_data), file=STDOUT)
     return 0 if len(json_data['errors']) == 0 else 1
 
 
@@ -360,7 +388,7 @@ def cmd_get_instances(_args):
         LOG.error('no response received: %s', msg)
         return 1
 
-    res = Result.from_broker(resp.result)
+    res = Result.from_brokertype(resp.result)
 
     if not res.success:
         msg = res.error if res.error else 'no reason given'
@@ -377,13 +405,13 @@ def cmd_get_instances(_args):
     # instances easier to comprehend than raw Broker data: turn it into Instance
     # objects, then render these JSON-friendly.
     try:
-        for inst in sorted([Instance.from_broker(inst) for inst in res.data]):
+        for inst in sorted([Instance.from_brokertype(inst) for inst in res.data]):
             json_data[inst.name] = inst.to_json_data()
             json_data[inst.name].pop('name')
     except TypeError as err:
         LOG.error('instance data invalid: %s', err)
 
-    print(json_dumps(json_data))
+    print(json_dumps(json_data), file=STDOUT)
     return 0
 
 
@@ -403,7 +431,7 @@ def cmd_get_nodes(_args):
         'errors': [],
     }
 
-    results = [Result.from_broker(broker_data) for broker_data in resp.results]
+    results = [Result.from_brokertype(broker_data) for broker_data in resp.results]
 
     for res in sorted(results):
         if not res.success:
@@ -424,7 +452,7 @@ def cmd_get_nodes(_args):
 
         # res.data is a NodeStatusVec
         try:
-            nstats = [NodeStatus.from_broker(nstat_data) for nstat_data in res.data]
+            nstats = [NodeStatus.from_brokertype(nstat_data) for nstat_data in res.data]
             for nstat in sorted(nstats):
                 # If either of the two role enums are "NONE", we make them
                 # None. That way they stay in the reporting, but are more easily
@@ -446,7 +474,7 @@ def cmd_get_nodes(_args):
             LOG.error('NodeStatus data invalid: %s', err)
             LOG.debug(traceback.format_exc())
 
-    print(json_dumps(json_data))
+    print(json_dumps(json_data), file=STDOUT)
     return 0 if len(json_data['errors']) == 0 else 1
 
 
@@ -488,7 +516,7 @@ def cmd_restart(args):
     # task to Python, for our error reporting it's up to us, and we want be
     # reproducible.
 
-    results = [Result.from_broker(broker_data) for broker_data in resp.results]
+    results = [Result.from_brokertype(broker_data) for broker_data in resp.results]
 
     for res in sorted(results):
         if not res.success and res.instance is None:
@@ -508,18 +536,19 @@ def cmd_restart(args):
             'error': 'result lacking node: {}'.format(res),
         })
 
-    print(json_dumps(json_data))
+    print(json_dumps(json_data), file=STDOUT)
     return 0 if len(json_data['errors']) == 0 else 1
 
 
 def cmd_stage_config_impl(args):
     """Internals of cmd_stage_config() to enable chaining with other commands.
 
-    Returns a tuple of exit code and any JSON data to show to the user/caller.
+    Returns a tuple of exit code, any JSON data to show to the user/caller, and
+    the created controller object, if any.
     """
     if not args.config or (args.config != '-' and not os.path.isfile(args.config)):
         LOG.error('please provide a cluster configuration file.')
-        return 1, None
+        return 1, None, None
 
     # We use a config parser to parse the cluster configuration. For instances,
     # we allow names without value to designate agents that connect to the
@@ -533,7 +562,7 @@ def cmd_stage_config_impl(args):
     cfp = configparser.ConfigParser(allow_no_value=True)
 
     if args.config == '-':
-        cfp.read_file(sys.stdin)
+        cfp.read_file(STDIN)
     else:
         cfp.read(args.config)
 
@@ -541,19 +570,19 @@ def cmd_stage_config_impl(args):
 
     if config is None:
         LOG.error('configuration has errors, not sending')
-        return 1, None
+        return 1, None, None
 
     controller = create_controller()
     if controller is None:
-        return 1, None
+        return 1, None, None
 
     resp, msg = controller.transact(StageConfigurationRequest,
                                     StageConfigurationResponse,
-                                    config.to_broker())
+                                    config.to_brokertype())
 
     if resp is None:
         LOG.error('no response received: %s', msg)
-        return 1, None
+        return 1, None, controller
 
     retval = 0
     json_data = {
@@ -562,7 +591,7 @@ def cmd_stage_config_impl(args):
     }
 
     for broker_data in resp.results:
-        res = Result.from_broker(broker_data)
+        res = Result.from_brokertype(broker_data)
 
         if not res.success:
             retval = 1
@@ -576,31 +605,31 @@ def cmd_stage_config_impl(args):
         if res.data:
             json_data['results']['id'] = res.data
 
-    return retval, json_data
+    return retval, json_data, controller
 
 
 def cmd_stage_config(args):
-    ret, json_data = cmd_stage_config_impl(args)
+    ret, json_data, _ = cmd_stage_config_impl(args)
 
     if json_data:
-        print(json_dumps(json_data))
+        print(json_dumps(json_data), file=STDOUT)
 
     return ret
 
 
 def cmd_deploy_config(args):
-    ret, json_data = cmd_stage_config_impl(args)
+    ret, json_data, controller = cmd_stage_config_impl(args)
 
     if ret != 0:
         if json_data:
-            print(json_dumps(json_data))
+            print(json_dumps(json_data), file=STDOUT)
         return ret
 
-    return cmd_deploy(args)
+    return cmd_deploy(args, controller=controller)
 
 
 def cmd_show_settings(_args):
-    CONFIG.write(sys.stdout)
+    CONFIG.write(STDOUT)
     return 0
 
 
@@ -616,6 +645,7 @@ def cmd_test_timeout(args):
         LOG.error('no response received: %s', msg)
         return 1
 
-    res = Result.from_broker(resp.result)
-    print(json_dumps({'success': res.success, 'error': res.error}))
+    res = Result.from_brokertype(resp.result)
+    print(json_dumps({'success': res.success, 'error': res.error}),
+          file=STDOUT)
     return 0
