@@ -350,7 +350,7 @@ static int get_ttl(unsigned char* abuf, int alen, int* ttl)
 
 /**
  * Called in response to ares_getaddrinfo requests. Builds a hostent structure from
- * the result data and sends it to the DNS manager via Addresult().
+ * the result data and sends it to the DNS manager via AddResult().
  */
 static void addrinfo_cb(void* arg, int status, int timeouts, struct ares_addrinfo* result)
 	{
@@ -536,14 +536,14 @@ static void query_cb(void* arg, int status, int timeouts, unsigned char* buf, in
 	}
 
 /**
- * Called when the c-ares socket changes state, whcih indicates that it's connected to
+ * Called when the c-ares socket changes state, which indicates that it's connected to
  * some source of data (either a host file or a DNS server). This indicates that we're
  * able to do lookups against c-ares now and should activate the IOSource.
  */
-static void sock_cb(void* data, int s, int read, int write)
+static void sock_cb(void* data, ares_socket_t s, int read, int write)
 	{
 	auto mgr = reinterpret_cast<DNS_Mgr*>(data);
-	mgr->RegisterSocket(s, read == 1, write == 1);
+	mgr->RegisterSocket((int)s, read == 1, write == 1);
 	}
 
 DNS_Mgr::DNS_Mgr(DNS_MgrMode arg_mode) : IOSource(true), mode(arg_mode)
@@ -642,7 +642,6 @@ void DNS_Mgr::InitSource()
 		servers.next = NULL;
 
 		auto dns_resolver_addr = IPAddr(dns_resolver);
-		struct sockaddr_storage ss = {0};
 
 		if ( dns_resolver_addr.GetFamily() == IPv4 )
 			{
@@ -651,12 +650,12 @@ void DNS_Mgr::InitSource()
 			}
 		else
 			{
-			struct sockaddr_in6* sa = (struct sockaddr_in6*)&ss;
-			sa->sin6_family = AF_INET6;
-			dns_resolver_addr.CopyIPv6(&sa->sin6_addr);
+			struct sockaddr_in6 sa = {0};
+			sa.sin6_family = AF_INET6;
+			dns_resolver_addr.CopyIPv6(&sa.sin6_addr);
 
 			servers.family = AF_INET6;
-			memcpy(&(servers.addr.addr6), &sa->sin6_addr, sizeof(ares_in6_addr));
+			memcpy(&(servers.addr.addr6), &sa.sin6_addr, sizeof(ares_in6_addr));
 			}
 
 		ares_set_servers(channel, &servers);
@@ -1625,6 +1624,8 @@ void TestDNS_Mgr::Process()
 
 TEST_CASE("dns_mgr priming" * doctest::skip(true))
 	{
+		// TODO: This test uses mkdtemp, which isn't available on Windows.
+#ifndef _MSC_VER
 	char prefix[] = "/tmp/zeek-unit-test-XXXXXX";
 	auto tmpdir = mkdtemp(prefix);
 
@@ -1670,6 +1671,7 @@ TEST_CASE("dns_mgr priming" * doctest::skip(true))
 	// Clean up cache file and the temp directory
 	unlink(mgr2.CacheFile().c_str());
 	rmdir(tmpdir);
+#endif
 	}
 
 TEST_CASE("dns_mgr alternate server" * doctest::skip(true))
