@@ -3,6 +3,7 @@
 #pragma once
 
 #include <algorithm>
+#include <cmath>
 #include <cstdint>
 #include <fstream>
 #include <memory>
@@ -179,6 +180,8 @@ public:
 	bool operator!=(const DictEntry& r) const { return ! Equal(r.GetKey(), r.key_size, r.hash); }
 	};
 
+using DictEntryVec = std::vector<detail::HashKey>;
+
 	} // namespace detail
 
 template <typename T> class DictIterator
@@ -214,6 +217,8 @@ public:
 		dict = that.dict;
 		curr = that.curr;
 		end = that.end;
+		ordered_iter = that.ordered_iter;
+
 		dict->IncrIters();
 		}
 
@@ -231,12 +236,14 @@ public:
 		dict = that.dict;
 		curr = that.curr;
 		end = that.end;
+		ordered_iter = that.ordered_iter;
+
 		dict->IncrIters();
 
 		return *this;
 		}
 
-	DictIterator(DictIterator&& that)
+	DictIterator(DictIterator&& that) noexcept
 		{
 		if ( this == &that )
 			return;
@@ -250,11 +257,12 @@ public:
 		dict = that.dict;
 		curr = that.curr;
 		end = that.end;
+		ordered_iter = that.ordered_iter;
 
 		that.dict = nullptr;
 		}
 
-	DictIterator& operator=(DictIterator&& that)
+	DictIterator& operator=(DictIterator&& that) noexcept
 		{
 		if ( this == &that )
 			return *this;
@@ -268,25 +276,64 @@ public:
 		dict = that.dict;
 		curr = that.curr;
 		end = that.end;
+		ordered_iter = that.ordered_iter;
 
 		that.dict = nullptr;
 
 		return *this;
 		}
 
-	reference operator*() { return *curr; }
-	reference operator*() const { return *curr; }
-	pointer operator->() { return curr; }
-	pointer operator->() const { return curr; }
+	reference operator*()
+		{
+		if ( dict->IsOrdered() )
+			{
+			// TODO: how does this work if ordered_iter == end(). LookupEntry will return a nullptr,
+			// which the dereference will fail on. That's undefined behavior, correct? Is that any
+			// different than if the unordered version returns a dereference of it's end?
+			auto e = dict->LookupEntry(*ordered_iter);
+			return *e;
+			}
+
+		return *curr;
+		}
+	reference operator*() const
+		{
+		if ( dict->IsOrdered() )
+			{
+			auto e = dict->LookupEntry(*ordered_iter);
+			return *e;
+			}
+
+		return *curr;
+		}
+	pointer operator->()
+		{
+		if ( dict->IsOrdered() )
+			return dict->LookupEntry(*ordered_iter);
+
+		return curr;
+		}
+	pointer operator->() const
+		{
+		if ( dict->IsOrdered() )
+			return dict->LookupEntry(*ordered_iter);
+
+		return curr;
+		}
 
 	DictIterator& operator++()
 		{
-		// The non-robust case is easy. Just advanced the current position forward until you find
-		// one isn't empty and isn't the end.
-		do
+		if ( dict->IsOrdered() )
+			++ordered_iter;
+		else
 			{
-			++curr;
-			} while ( curr != end && curr->Empty() );
+			// The non-robust case is easy. Just advance the current position forward until you
+			// find one isn't empty and isn't the end.
+			do
+				{
+				++curr;
+				} while ( curr != end && curr->Empty() );
+			}
 
 		return *this;
 		}
@@ -298,7 +345,17 @@ public:
 		return temp;
 		}
 
-	bool operator==(const DictIterator& that) const { return curr == that.curr; }
+	bool operator==(const DictIterator& that) const
+		{
+		if ( dict != that.dict )
+			return false;
+
+		if ( dict->IsOrdered() )
+			return ordered_iter == that.ordered_iter;
+
+		return curr == that.curr;
+		}
+
 	bool operator!=(const DictIterator& that) const { return ! (*this == that); }
 
 private:
@@ -307,10 +364,21 @@ private:
 	DictIterator(const Dictionary<T>* d, detail::DictEntry<T>* begin, detail::DictEntry<T>* end)
 		: curr(begin), end(end)
 		{
+		// Cast away the constness so that the number of iterators can be modified in the
+		// dictionary. This does violate the constness guarantees of const-begin()/end() and
+		// cbegin()/cend(), but we're not modifying the actual data in the collection, just a
+		// counter in the wrapper of the collection.
+		dict = const_cast<Dictionary<T>*>(d);
+
 		// Make sure that we're starting on a non-empty element.
 		while ( curr != end && curr->Empty() )
 			++curr;
 
+		dict->IncrIters();
+		}
+
+	DictIterator(const Dictionary<T>* d, detail::DictEntryVec::iterator iter) : ordered_iter(iter)
+		{
 		// Cast away the constness so that the number of iterators can be modified in the
 		// dictionary. This does violate the constness guarantees of const-begin()/end() and
 		// cbegin()/cend(), but we're not modifying the actual data in the collection, just a
@@ -322,6 +390,7 @@ private:
 	Dictionary<T>* dict = nullptr;
 	detail::DictEntry<T>* curr = nullptr;
 	detail::DictEntry<T>* end = nullptr;
+	detail::DictEntryVec::iterator ordered_iter;
 	};
 
 template <typename T> class RobustDictIterator
@@ -348,9 +417,48 @@ public:
 		curr = dict->GetNextRobustIteration(this);
 		}
 
-	RobustDictIterator(const RobustDictIterator& other) : curr(nullptr)
+	RobustDictIterator(const RobustDictIterator& other) : curr(nullptr), dict(nullptr)
 		{
+		*this = other;
+		}
+
+	RobustDictIterator(RobustDictIterator&& other) noexcept : curr(nullptr), dict(nullptr)
+		{
+		*this = other;
+		}
+
+	~RobustDictIterator() { Complete(); }
+
+	reference operator*() { return curr; }
+	pointer operator->() { return &curr; }
+
+	RobustDictIterator& operator++()
+		{
+		curr = dict->GetNextRobustIteration(this);
+		return *this;
+		}
+
+	RobustDictIterator operator++(int)
+		{
+		auto temp(*this);
+		++*this;
+		return temp;
+		}
+
+	RobustDictIterator& operator=(const RobustDictIterator& other)
+		{
+		if ( this == &other )
+			return *this;
+
+		delete inserted;
+		inserted = nullptr;
+
+		delete visited;
+		visited = nullptr;
+
 		dict = nullptr;
+		curr.Clear();
+		next = -1;
 
 		if ( other.dict )
 			{
@@ -372,11 +480,21 @@ public:
 
 			curr = other.curr;
 			}
+
+		return *this;
 		}
 
-	RobustDictIterator(RobustDictIterator&& other) : curr(nullptr)
+	RobustDictIterator& operator=(RobustDictIterator&& other) noexcept
 		{
+		delete inserted;
+		inserted = nullptr;
+
+		delete visited;
+		visited = nullptr;
+
 		dict = nullptr;
+		curr.Clear();
+		next = -1;
 
 		if ( other.dict )
 			{
@@ -393,26 +511,10 @@ public:
 
 			curr = std::move(other.curr);
 			}
-		}
 
-	~RobustDictIterator() { Complete(); }
-
-	reference operator*() { return curr; }
-	pointer operator->() { return &curr; }
-
-	RobustDictIterator& operator++()
-		{
-		curr = dict->GetNextRobustIteration(this);
 		return *this;
 		}
 
-	RobustDictIterator operator++(int)
-		{
-		auto temp(*this);
-		++*this;
-		return temp;
-		}
-
 	bool operator==(const RobustDictIterator& that) const { return curr == that.curr; }
 	bool operator!=(const RobustDictIterator& that) const { return ! (*this == that); }
 
@@ -471,14 +573,14 @@ public:
 		{
 		if ( initial_size > 0 )
 			{
-			// If an initial size is speicified, init the table right away. Otherwise wait until the
+			// If an initial size is specified, init the table right away. Otherwise wait until the
 			// first insertion to init.
-			log2_buckets = Log2(initial_size);
+			SetLog2Buckets(static_cast<uint16_t>(std::log2(initial_size)));
 			Init();
 			}
 
 		if ( ordering == ORDERED )
-			order = new std::vector<detail::DictEntry<T>>;
+			order = std::make_unique<std::vector<detail::HashKey>>();
 		}
 
 	~Dictionary() { Clear(); }
@@ -495,12 +597,10 @@ public:
 
 	T* Lookup(const void* key, int key_size, detail::hash_t h) const
 		{
-		// Look up possibly modifies the entry. Why? if the entry is found but not positioned
-		// according to the current dict (so it's before SizeUp), it will be moved to the right
-		// position so next lookup is fast.
-		Dictionary* d = const_cast<Dictionary*>(this);
-		int position = d->LookupIndex(key, key_size, h);
-		return position >= 0 ? table[position].value : nullptr;
+		if ( auto e = LookupEntry(key, key_size, h) )
+			return e->value;
+
+		return nullptr;
 		}
 
 	T* Lookup(const char* key) const
@@ -545,7 +645,7 @@ public:
 			auto loc = detail::GetCurrentLocation();
 			reporter->RuntimeError(&loc,
 			                       "Attempted to create DictEntry with excessively large key, "
-			                       "truncating key (%" PRIu64 " > %d)",
+			                       "truncating key (%" PRIu64 " > %u)",
 			                       key_size, detail::DictEntry<T>::MAX_KEY_SIZE);
 			}
 
@@ -561,13 +661,6 @@ public:
 			if ( ! copy_key )
 				delete[](char*) key;
 
-			if ( order )
-				{ // set new v to order too.
-				auto it = std::find(order->begin(), order->end(), table[position]);
-				ASSERT(it != order->end());
-				it->value = val;
-				}
-
 			if ( iterators && ! iterators->empty() )
 				// need to set new v for iterators too.
 				for ( auto c : *iterators )
@@ -595,12 +688,15 @@ public:
 						"Dictionary::Insert() possibly caused iterator invalidation");
 				}
 
-			// Allocate memory for key if necesary. Key is updated to reflect internal key if
+			// Do this before the actual insertion since creating the DictEntry is going to delete
+			// the key data. We need a copy of it first.
+			if ( order )
+				order->emplace_back(detail::HashKey{key, static_cast<size_t>(key_size), hash});
+
+			// Allocate memory for key if necessary. Key is updated to reflect internal key if
 			// necessary.
 			detail::DictEntry<T> entry(key, key_size, hash, val, insert_distance, copy_key);
 			InsertRelocateAndAdjust(entry, insert_position);
-			if ( order )
-				order->push_back(entry);
 
 			num_entries++;
 			cum_entries++;
@@ -670,7 +766,16 @@ public:
 		ASSERT(num_entries >= 0);
 		// e is about to be invalid. remove it from all references.
 		if ( order )
-			order->erase(std::remove(order->begin(), order->end(), entry), order->end());
+			{
+			for ( auto it = order->begin(); it != order->end(); ++it )
+				{
+				if ( it->Equal(key, key_size, hash) )
+					{
+					it = order->erase(it);
+					break;
+					}
+				}
+			}
 
 		T* v = entry.value;
 		entry.Clear();
@@ -718,10 +823,13 @@ public:
 		{
 		if ( ! order || n < 0 || n >= Length() )
 			return nullptr;
-		detail::DictEntry<T> entry = (*order)[n];
-		key = entry.GetKey();
-		key_size = entry.key_size;
-		return entry.value;
+
+		auto& hk = order->at(n);
+		auto entry = Lookup(&hk);
+
+		key = hk.Key();
+		key_size = hk.Size();
+		return entry;
 		}
 
 	T* NthEntry(int n, const char*& key) const
@@ -750,10 +858,8 @@ public:
 			}
 
 		if ( order )
-			{
-			delete order;
-			order = nullptr;
-			}
+			order.reset();
+
 		if ( iterators )
 			{
 			delete iterators;
@@ -768,13 +874,8 @@ public:
 		}
 
 	/// The capacity of the table, Buckets + Overflow Size.
-	int Capacity(bool expected = false) const
-		{
-		int capacity = (1 << log2_buckets) + (log2_buckets + 0);
-		if ( expected )
-			return capacity;
-		return table ? capacity : 0;
-		}
+	int Capacity() const { return table ? bucket_capacity : 0; }
+	int ExpectedCapacity() const { return bucket_capacity; }
 
 	// Debugging
 #define DUMPIF(f)                                                                                  \
@@ -924,7 +1025,7 @@ public:
 		if ( binary )
 			{
 			char key = char(random() % 26) + 'A';
-			sprintf(key_file, "%d.%d-%c.key", Length(), max_distance, key);
+			snprintf(key_file, 100, "%d.%d-%c.key", Length(), max_distance, key);
 			std::ofstream f(key_file, std::ios::binary | std::ios::out | std::ios::trunc);
 			for ( int idx = 0; idx < Capacity(); idx++ )
 				if ( ! table[idx].Empty() )
@@ -937,7 +1038,7 @@ public:
 		else
 			{
 			char key = char(random() % 26) + 'A';
-			sprintf(key_file, "%d.%d-%d.ckey", Length(), max_distance, key);
+			snprintf(key_file, 100, "%d.%d-%d.ckey", Length(), max_distance, key);
 			std::ofstream f(key_file, std::ios::out | std::ios::trunc);
 			for ( int idx = 0; idx < Capacity(); idx++ )
 				if ( ! table[idx].Empty() )
@@ -959,12 +1060,48 @@ public:
 	using reverse_iterator = std::reverse_iterator<iterator>;
 	using const_reverse_iterator = std::reverse_iterator<const_iterator>;
 
-	iterator begin() { return {this, table, table + Capacity()}; }
-	iterator end() { return {this, table + Capacity(), table + Capacity()}; }
-	const_iterator begin() const { return {this, table, table + Capacity()}; }
-	const_iterator end() const { return {this, table + Capacity(), table + Capacity()}; }
-	const_iterator cbegin() { return {this, table, table + Capacity()}; }
-	const_iterator cend() { return {this, table + Capacity(), table + Capacity()}; }
+	iterator begin()
+		{
+		if ( IsOrdered() )
+			return {this, order->begin()};
+
+		return {this, table, table + Capacity()};
+		}
+	iterator end()
+		{
+		if ( IsOrdered() )
+			return {this, order->end()};
+
+		return {this, table + Capacity(), table + Capacity()};
+		}
+	const_iterator begin() const
+		{
+		if ( IsOrdered() )
+			return {this, order->begin()};
+
+		return {this, table, table + Capacity()};
+		}
+	const_iterator end() const
+		{
+		if ( IsOrdered() )
+			return {this, order->end()};
+
+		return {this, table + Capacity(), table + Capacity()};
+		}
+	const_iterator cbegin()
+		{
+		if ( IsOrdered() )
+			return {this, order->begin()};
+
+		return {this, table, table + Capacity()};
+		}
+	const_iterator cend()
+		{
+		if ( IsOrdered() )
+			return {this, order->end()};
+
+		return {this, table + Capacity(), table + Capacity()};
+		}
 
 	RobustDictIterator<T> begin_robust() { return MakeRobustIterator(); }
 	RobustDictIterator<T> end_robust() { return RobustDictIterator<T>(); }
@@ -973,25 +1110,18 @@ private:
 	friend zeek::DictIterator<T>;
 	friend zeek::RobustDictIterator<T>;
 
-	/// Buckets of the table, not including overflow size.
-	int Buckets(bool expected = false) const
+	void SetLog2Buckets(int value)
 		{
-		int buckets = (1 << log2_buckets);
-		if ( expected )
-			return buckets;
-		return table ? buckets : 0;
+		log2_buckets = value;
+		bucket_count = 1 << log2_buckets;
+		bucket_capacity = (1 << log2_buckets) + log2_buckets;
 		}
 
+	/// Buckets of the table, not including overflow size.
+	int Buckets() const { return table ? bucket_count : 0; }
+
 	// bucket math
-	int Log2(int num) const
-		{
-		int i = 0;
-		while ( num >>= 1 )
-			i++;
-		return i;
-		}
-
-	int ThresholdEntries() const
+	uint32_t ThresholdEntries() const
 		{
 		// Increase the size of the dictionary when it is 75% full. However, when the dictionary
 		// is small ( bucket_capacity <= 2^3+3=11 elements ), only resize it when it's 100% full.
@@ -1047,7 +1177,8 @@ private:
 		{
 		ASSERT(bucket >= 0 && bucket < Buckets());
 		int i = bucket;
-		while ( i < Capacity() && ! table[i].Empty() && BucketByPosition(i) <= bucket )
+		int current_cap = Capacity();
+		while ( i < current_cap && ! table[i].Empty() && BucketByPosition(i) <= bucket )
 			i++;
 		return i;
 		}
@@ -1074,7 +1205,8 @@ private:
 
 		int bucket = BucketByPosition(position);
 		int i = position;
-		while ( i < Capacity() && ! table[i].Empty() && BucketByPosition(i) == bucket )
+		int current_cap = Capacity();
+		while ( i < current_cap && ! table[i].Empty() && BucketByPosition(i) == bucket )
 			i++; // stop just over the tail.
 
 		return i - 1;
@@ -1099,10 +1231,11 @@ private:
 		{
 		ASSERT(table && -1 <= position && position < Capacity());
 
+		int current_cap = Capacity();
 		do
 			{
 			position++;
-			} while ( position < Capacity() && table[position].Empty() );
+			} while ( position < current_cap && table[position].Empty() );
 
 		return position;
 		}
@@ -1110,7 +1243,7 @@ private:
 	void Init()
 		{
 		ASSERT(! table);
-		table = (detail::DictEntry<T>*)malloc(sizeof(detail::DictEntry<T>) * Capacity(true));
+		table = (detail::DictEntry<T>*)malloc(sizeof(detail::DictEntry<T>) * ExpectedCapacity());
 		for ( int i = Capacity() - 1; i >= 0; i-- )
 			table[i].SetEmpty();
 		}
@@ -1118,7 +1251,8 @@ private:
 	// Lookup
 	int LinearLookupIndex(const void* key, int key_size, detail::hash_t hash) const
 		{
-		for ( int i = 0; i < Capacity(); i++ )
+		auto current_cap = Capacity();
+		for ( int i = 0; i < current_cap; i++ )
 			if ( ! table[i].Empty() && table[i].Equal((const char*)key, key_size, hash) )
 				return i;
 		return -1;
@@ -1169,7 +1303,7 @@ private:
 		// not found
 #ifdef ZEEK_DICT_DEBUG
 		if ( linear_position >= 0 )
-			{ // different. stop and try to see whats happending.
+			{ // different. stop and try to see whats happening.
 			ASSERT(false);
 			// rerun the function in debugger to track down the bug.
 			LookupIndex(key, key_size, hash);
@@ -1432,7 +1566,8 @@ private:
 	void SizeUp()
 		{
 		int prev_capacity = Capacity();
-		log2_buckets++;
+		SetLog2Buckets(log2_buckets + 1);
+
 		int capacity = Capacity();
 		table = (detail::DictEntry<T>*)realloc(table, capacity * sizeof(detail::DictEntry<T>));
 		for ( int i = prev_capacity; i < capacity; i++ )
@@ -1453,6 +1588,35 @@ private:
 		space_distance_samples = 0;
 		}
 
+	/**
+	 * Retrieves a pointer to a full DictEntry in the table based on a hash key.
+	 *
+	 * @param key the key to lookup.
+	 * @return A pointer to the entry or a nullptr if no entry has a matching key.
+	 */
+	detail::DictEntry<T>* LookupEntry(const detail::HashKey& key)
+		{
+		return LookupEntry(key.Key(), key.Size(), key.Hash());
+		}
+
+	/**
+	 * Retrieves a pointer to a full DictEntry in the table based on key data.
+	 *
+	 * @param key the key to lookup
+	 * @param key_size the size of the key data
+	 * @param h a hash of the key data.
+	 * @return A pointer to the entry or a nullptr if no entry has a matching key.
+	 */
+	detail::DictEntry<T>* LookupEntry(const void* key, int key_size, detail::hash_t h) const
+		{
+		// Look up possibly modifies the entry. Why? if the entry is found but not positioned
+		// according to the current dict (so it's before SizeUp), it will be moved to the right
+		// position so next lookup is fast.
+		Dictionary* d = const_cast<Dictionary*>(this);
+		int position = d->LookupIndex(key, key_size, h);
+		return position >= 0 ? &(table[position]) : nullptr;
+		}
+
 	bool HaveOnlyRobustIterators() const
 		{
 		return (num_iterators == 0) || ((iterators ? iterators->size() : 0) == num_iterators);
@@ -1460,6 +1624,10 @@ private:
 
 	RobustDictIterator<T> MakeRobustIterator()
 		{
+		if ( IsOrdered() )
+			reporter->InternalError(
+				"RobustIterators are not currently supported for ordered dictionaries");
+
 		if ( ! iterators )
 			iterators = new std::vector<RobustDictIterator<T>*>;
 
@@ -1468,15 +1636,17 @@ private:
 
 	detail::DictEntry<T> GetNextRobustIteration(RobustDictIterator<T>* iter)
 		{
-		// If there are any inserted entries, return them first.
-		// That keeps the list small and helps avoiding searching
-		// a large list when deleting an entry.
+		// If there's no table in the dictionary, then the iterator needs to be
+		// cleaned up because it's not pointing at anything.
 		if ( ! table )
 			{
 			iter->Complete();
 			return detail::DictEntry<T>(nullptr); // end of iteration
 			}
 
+		// If there are any inserted entries, return them first.
+		// That keeps the list small and helps avoiding searching
+		// a large list when deleting an entry.
 		if ( iter->inserted && ! iter->inserted->empty() )
 			{
 			// Return the last one. Order doesn't matter,
@@ -1486,6 +1656,7 @@ private:
 			return e;
 			}
 
+		// First iteration.
 		if ( iter->next < 0 )
 			iter->next = Next(-1);
 
@@ -1536,24 +1707,26 @@ private:
 	void IncrIters() { ++num_iterators; }
 	void DecrIters() { --num_iterators; }
 
-	// alligned on 8-bytes with 4-leading bytes. 7*8=56 bytes a dictionary.
+	// aligned on 8-bytes with 4-leading bytes. 7*8=56 bytes a dictionary.
 
 	// when sizeup but the current mapping is in progress. the current mapping will be ignored
 	// as it will be remapped to new dict size anyway. however, the missed count is recorded
 	// for lookup. if position not found for a key in the position of dict of current size, it
 	// still could be in the position of dict of previous N sizes.
-	unsigned char remaps = 0;
-	unsigned char log2_buckets = 0;
+	uint16_t remaps = 0;
+	uint16_t log2_buckets = 0;
+	uint32_t bucket_capacity = 1;
+	uint32_t bucket_count = 1;
 
 	// Pending number of iterators on the Dict, including both robust and non-robust.
 	// This is used to avoid remapping if there are any active iterators.
-	unsigned short num_iterators = 0;
+	uint16_t num_iterators = 0;
 
 	// The last index to be remapped.
-	int remap_end = -1;
+	int32_t remap_end = -1;
 
-	int num_entries = 0;
-	int max_entries = 0;
+	uint32_t num_entries = 0;
+	uint32_t max_entries = 0;
 	uint64_t cum_entries = 0;
 	uint32_t space_distance_samples = 0;
 	// how far the space is
@@ -1563,8 +1736,10 @@ private:
 	detail::DictEntry<T>* table = nullptr;
 	std::vector<RobustDictIterator<T>*>* iterators = nullptr;
 
-	// Order means the order of insertion. means no deletion until exit. will be inefficient.
-	std::vector<detail::DictEntry<T>>* order = nullptr;
+	// Ordered dictionaries keep the order based on some criteria, by default the order of
+	// insertion. We only store a copy of the keys here for memory savings and for safety
+	// around reallocs and such.
+	std::unique_ptr<detail::DictEntryVec> order;
 	};
 
 template <typename T> using PDict = Dictionary<T>;
