@@ -5,8 +5,10 @@
 // Zeek statements.
 
 #include "zeek/Dict.h"
+#include "zeek/Expr.h"
 #include "zeek/ID.h"
 #include "zeek/StmtBase.h"
+#include "zeek/Type.h"
 #include "zeek/ZeekList.h"
 
 namespace zeek::detail
@@ -191,6 +193,7 @@ public:
 
 protected:
 	friend class ZAMCompiler;
+	friend class CPPCompile;
 
 	int DefaultCaseIndex() const { return default_case_idx; }
 	const auto& ValueMap() const { return case_label_value_map; }
@@ -442,7 +445,7 @@ public:
 	// Optimization-related:
 	StmtPtr Duplicate() override;
 
-	// Constructor used for duplication, when we've already done
+	// Constructor used internally, for when we've already done
 	// all of the type-checking.
 	ReturnStmt(ExprPtr e, bool ignored);
 
@@ -522,7 +525,7 @@ protected:
 class NullStmt final : public Stmt
 	{
 public:
-	NullStmt() : Stmt(STMT_NULL) { }
+	NullStmt(bool arg_is_directive = false);
 
 	ValPtr Exec(Frame* f, StmtFlowType& flow) override;
 	bool IsPure() const override;
@@ -533,23 +536,142 @@ public:
 
 	// Optimization-related:
 	StmtPtr Duplicate() override { return SetSucc(new NullStmt()); }
+
+	// Returns true if this NullStmt represents a directive (@if..., @else, @endif)
+	bool IsDirective() const { return is_directive; };
+
+private:
+	bool is_directive;
+	};
+
+// A helper class for tracking all of the information associated with
+// a "when" statement, and constructing the necessary components in support
+// of lambda-style captures.
+class WhenInfo
+	{
+public:
+	// Takes ownership of the CaptureList, which if nil signifies
+	// old-style frame semantics.
+	WhenInfo(ExprPtr cond, FuncType::CaptureList* cl, bool is_return);
+
+	// Constructor used by script optimization to create a stub.
+	WhenInfo(bool is_return);
+
+	~WhenInfo() { delete cl; }
+
+	void AddBody(StmtPtr arg_s) { s = std::move(arg_s); }
+
+	void AddTimeout(ExprPtr arg_timeout, StmtPtr arg_timeout_s)
+		{
+		timeout = std::move(arg_timeout);
+		timeout_s = std::move(arg_timeout_s);
+		}
+
+	// Complete construction of the associated internals, including
+	// the (complex) lambda used to access the different elements of
+	// the statement.  The optional argument is only for generating
+	// error messages.
+	void Build(StmtPtr ws = nullptr);
+
+	// This is available after a call to Build().
+	const LambdaExprPtr& Lambda() const { return lambda; }
+
+	// Instantiate a new instance, either by evaluating the associated
+	// lambda, or directly using the given function value (for compiled
+	// code).
+	void Instantiate(Frame* f);
+	void Instantiate(ValPtr func);
+
+	// For old-style semantics, the following simply return the
+	// individual "when" components.  For capture semantics, however,
+	// these instead return different invocations of a lambda that
+	// manages the captures.
+	ExprPtr Cond();
+	StmtPtr WhenBody();
+
+	ExprPtr TimeoutExpr() const { return timeout; }
+	double TimeoutVal(Frame* f);
+
+	StmtPtr TimeoutStmt();
+
+	FuncType::CaptureList* Captures() { return cl; }
+
+	bool IsReturn() const { return is_return; }
+
+	// The locals and globals used in the conditional expression
+	// (other than newly introduced locals), necessary for registering
+	// the associated triggers for when their values change.
+	const IDSet& WhenExprLocals() const { return when_expr_locals; }
+	const IDSet& WhenExprGlobals() const { return when_expr_globals; }
+
+private:
+	// True if the "when" statement corresponds to old-style deprecated
+	// semantics (no captures, but needing captures).  Also generates
+	// the corresponding deprecation warnings, which are associated
+	// with "ws".
+	bool IsDeprecatedSemantics(StmtPtr ws);
+
+	// Build those elements we'll need for invoking our lambda.
+	void BuildInvokeElems();
+
+	ExprPtr cond;
+	StmtPtr s;
+	ExprPtr timeout;
+	StmtPtr timeout_s;
+	FuncType::CaptureList* cl;
+
+	bool is_return = false;
+
+	// The name of parameter passed ot the lambda.
+	std::string lambda_param_id;
+
+	// The expression for constructing the lambda, and its type.
+	LambdaExprPtr lambda;
+	FuncTypePtr lambda_ft;
+
+	// The current instance of the lambda.  Created by Instantiate(),
+	// for immediate use via calls to Cond() etc.
+	ConstExprPtr curr_lambda;
+
+	// Arguments to use when calling the lambda to either evaluate
+	// the conditional, or execute the body or the timeout statement.
+	ListExprPtr invoke_cond;
+	ListExprPtr invoke_s;
+	ListExprPtr invoke_timeout;
+
+	// Helper expressions for calling the lambda / testing within it.
+	ConstExprPtr one_const;
+	ConstExprPtr two_const;
+	ConstExprPtr three_const;
+
+	IDSet when_expr_locals;
+	IDSet when_expr_globals;
+
+	// Locals introduced via "local" in the "when" clause itself.
+	IDSet when_new_locals;
+
+	// Used for identifying deprecated instances.  Holds all of the local
+	// variables in the scope prior to parsing the "when" statement.
+	std::map<std::string, IDPtr, std::less<>> prior_vars;
 	};
 
 class WhenStmt final : public Stmt
 	{
 public:
-	// s2 is null if no timeout block given.
-	WhenStmt(ExprPtr cond, StmtPtr s1, StmtPtr s2, ExprPtr timeout, bool is_return);
+	// The constructor takes ownership of the WhenInfo object.
+	WhenStmt(WhenInfo* wi);
 	~WhenStmt() override;
 
 	ValPtr Exec(Frame* f, StmtFlowType& flow) override;
 	bool IsPure() const override;
 
-	const Expr* Cond() const { return cond.get(); }
-	const Stmt* Body() const { return s1.get(); }
-	const Expr* TimeoutExpr() const { return timeout.get(); }
-	const Stmt* TimeoutBody() const { return s2.get(); }
-	bool IsReturn() const { return is_return; }
+	ExprPtr Cond() const { return wi->Cond(); }
+	StmtPtr Body() const { return wi->WhenBody(); }
+	ExprPtr TimeoutExpr() const { return wi->TimeoutExpr(); }
+	StmtPtr TimeoutBody() const { return wi->TimeoutStmt(); }
+	bool IsReturn() const { return wi->IsReturn(); }
+
+	const WhenInfo* Info() const { return wi; }
 
 	void StmtDescribe(ODesc* d) const override;
 
@@ -561,12 +683,8 @@ public:
 
 	bool IsReduced(Reducer* c) const override;
 
-protected:
-	ExprPtr cond;
-	StmtPtr s1;
-	StmtPtr s2;
-	ExprPtr timeout;
-	bool is_return;
+private:
+	WhenInfo* wi;
 	};
 
 // Internal statement used for inlining.  Executes a block and stops
