@@ -339,6 +339,10 @@ public:
 	// is not pure or is empty.
 	const TypePtr& GetPureType() const { return pure_type; }
 
+	// Retrospectively instantiates an underlying pure type, if in
+	// fact each element has the same type.
+	void CheckPure();
+
 	// True if all of the types match t, false otherwise.  If
 	// is_init is true, then the matching is done in the context
 	// of an initialization.
@@ -446,7 +450,7 @@ public:
 		std::string deprecation_msg;
 		RecordTypePtr args;
 		// Maps from parameter index in canonical prototype to
-		// parameter index in this alternate prorotype.
+		// parameter index in this alternate prototype.
 		std::map<int, int> offsets;
 		};
 
@@ -579,6 +583,8 @@ public:
 	TypeDecl(const TypeDecl& other);
 	~TypeDecl();
 
+	TypeDecl& operator=(const TypeDecl& other);
+
 	const detail::AttrPtr& GetAttr(detail::AttrTag a) const
 		{
 		return attrs ? attrs->Find(a) : detail::Attr::nil;
@@ -719,12 +725,12 @@ protected:
 	std::vector<bool> managed_fields;
 
 	// Number of fields in the type.
-	int num_fields;
+	int num_fields = 0;
 
 	// Number of fields in the type when originally declared.
-	int num_orig_fields;
+	int num_orig_fields = 0;
 
-	type_decl_list* types;
+	type_decl_list* types = nullptr;
 	std::set<std::string> field_ids;
 	};
 
@@ -781,7 +787,7 @@ public:
 
 	// The value of this name is set to val. Once a value has been
 	// explicitly assigned using this method, no further names can be
-	// added that aren't likewise explicitly initalized.
+	// added that aren't likewise explicitly initialized.
 	void AddName(const std::string& module_name, const char* name, zeek_int_t val, bool is_export,
 	             detail::Expr* deprecation = nullptr, bool from_redef = false);
 
@@ -925,7 +931,13 @@ TypePtr merge_types(const TypePtr& t1, const TypePtr& t2);
 // Given a list of expressions, returns a (ref'd) type reflecting
 // a merged type consistent across all of them, or nil if this
 // cannot be done.
-TypePtr merge_type_list(detail::ListExpr* elements);
+[[deprecated("Remove in v6.1. Use maximal_type() if possible. See GH-2604.")]] TypePtr
+merge_type_list(detail::ListExpr* elements);
+
+// Given a list of expressions, returns the maximal type consistent across
+// all of them, or nil if this cannot be done.  "Maximal" incorporates
+// notions of arithmetic coercion, but otherwise requires type-equivalence.
+TypePtr maximal_type(detail::ListExpr* elements);
 
 // Given an expression, infer its type when used for an initialization.
 TypePtr init_type(const detail::ExprPtr& init);
@@ -984,13 +996,13 @@ inline bool IsFunc(TypeTag t)
 	return (t == TYPE_FUNC);
 	}
 
-// True if the given type type is a vector.
+// True if the given type tag is a vector.
 inline bool IsVector(TypeTag t)
 	{
 	return (t == TYPE_VECTOR);
 	}
 
-// True if the given type type is a string.
+// True if the given type tag is a string.
 inline bool IsString(TypeTag t)
 	{
 	return (t == TYPE_STRING);
