@@ -5,6 +5,7 @@
 #include <binpac.h>
 #include <algorithm>
 
+#include "zeek/3rdparty/doctest.h"
 #include "zeek/Event.h"
 #include "zeek/ZeekString.h"
 #include "zeek/analyzer/Manager.h"
@@ -480,11 +481,11 @@ void Analyzer::PreventChildren(zeek::Tag tag)
 bool Analyzer::HasChildAnalyzer(zeek::Tag tag)
 	{
 	LOOP_OVER_CHILDREN(i)
-	if ( (*i)->tag == tag )
+	if ( (*i)->tag == tag && ! ((*i)->removing || (*i)->finished) )
 		return true;
 
 	LOOP_OVER_GIVEN_CHILDREN(i, new_children)
-	if ( (*i)->tag == tag )
+	if ( (*i)->tag == tag && ! ((*i)->removing || (*i)->finished) )
 		return true;
 
 	return false;
@@ -492,7 +493,7 @@ bool Analyzer::HasChildAnalyzer(zeek::Tag tag)
 
 Analyzer* Analyzer::FindChild(ID arg_id)
 	{
-	if ( id == arg_id )
+	if ( id == arg_id && ! (removing || finished) )
 		return this;
 
 	LOOP_OVER_CHILDREN(i)
@@ -514,7 +515,7 @@ Analyzer* Analyzer::FindChild(ID arg_id)
 
 Analyzer* Analyzer::FindChild(zeek::Tag arg_tag)
 	{
-	if ( tag == arg_tag )
+	if ( tag == arg_tag && ! (removing || finished) )
 		return this;
 
 	LOOP_OVER_CHILDREN(i)
@@ -601,7 +602,7 @@ void Analyzer::RemoveSupportAnalyzer(SupportAnalyzer* analyzer)
 
 	// We mark the analyzer as being removed here, which will prevent it
 	// from being used further. However, we don't actually delete it
-	// before the parent gets destroyed. While we woulc do that, it's a
+	// before the parent gets destroyed. While we could do that, it's a
 	// bit tricky to do at the right time and it doesn't seem worth the
 	// trouble.
 	analyzer->removing = true;
@@ -678,6 +679,24 @@ void Analyzer::FlipRoles()
 	resp_supporters = tmp;
 	}
 
+void Analyzer::EnqueueAnalyzerConfirmationInfo(const zeek::Tag& arg_tag)
+	{
+	static auto info_type = zeek::id::find_type<RecordType>("AnalyzerConfirmationInfo");
+	static auto info_c_idx = info_type->FieldOffset("c");
+	static auto info_aid_idx = info_type->FieldOffset("aid");
+
+	auto info = make_intrusive<RecordVal>(info_type);
+	info->Assign(info_c_idx, ConnVal());
+	info->Assign(info_aid_idx, val_mgr->Count(id));
+
+	event_mgr.Enqueue(analyzer_confirmation_info, arg_tag.AsVal(), info);
+	}
+
+void Analyzer::EnqueueAnalyzerConfirmation(const zeek::Tag& arg_tag)
+	{
+	event_mgr.Enqueue(analyzer_confirmation, ConnVal(), arg_tag.AsVal(), val_mgr->Count(id));
+	}
+
 void Analyzer::AnalyzerConfirmation(zeek::Tag arg_tag)
 	{
 	if ( analyzer_confirmed )
@@ -685,30 +704,39 @@ void Analyzer::AnalyzerConfirmation(zeek::Tag arg_tag)
 
 	analyzer_confirmed = true;
 
-	if ( ! analyzer_confirmation )
-		return;
+	const auto& effective_tag = arg_tag ? arg_tag : tag;
 
-	const auto& tval = arg_tag ? arg_tag.AsVal() : tag.AsVal();
-	event_mgr.Enqueue(analyzer_confirmation, ConnVal(), tval, val_mgr->Count(id));
+	if ( analyzer_confirmation_info )
+		EnqueueAnalyzerConfirmationInfo(effective_tag);
+
+	if ( analyzer_confirmation )
+		EnqueueAnalyzerConfirmation(effective_tag);
 	}
 
-void Analyzer::AnalyzerViolation(const char* reason, const char* data, int len, zeek::Tag arg_tag)
+void Analyzer::EnqueueAnalyzerViolationInfo(const char* reason, const char* data, int len,
+                                            const zeek::Tag& arg_tag)
 	{
-	if ( ! analyzer_violation )
-		return;
+	static auto info_type = zeek::id::find_type<RecordType>("AnalyzerViolationInfo");
+	static auto info_reason_idx = info_type->FieldOffset("reason");
+	static auto info_c_idx = info_type->FieldOffset("c");
+	static auto info_aid_idx = info_type->FieldOffset("aid");
+	static auto info_data_idx = info_type->FieldOffset("data");
 
+	auto info = zeek::make_intrusive<RecordVal>(info_type);
+	info->Assign(info_reason_idx, make_intrusive<StringVal>(reason));
+	info->Assign(info_c_idx, ConnVal());
+	info->Assign(info_aid_idx, val_mgr->Count(id));
+	if ( data && len )
+		info->Assign(info_data_idx, make_intrusive<StringVal>(len, data));
+
+	event_mgr.Enqueue(analyzer_violation_info, arg_tag.AsVal(), info);
+	}
+
+void Analyzer::EnqueueAnalyzerViolation(const char* reason, const char* data, int len,
+                                        const zeek::Tag& arg_tag)
+	{
 	StringValPtr r;
 
-	++analyzer_violations;
-
-	if ( analyzer_violations > BifConst::max_analyzer_violations )
-		{
-		if ( analyzer_violations == BifConst::max_analyzer_violations + 1 )
-			Weird("too_many_analyzer_violations");
-
-		return;
-		}
-
 	if ( data && len )
 		{
 		const char* tmp = util::copy_string(reason);
@@ -719,8 +747,29 @@ void Analyzer::AnalyzerViolation(const char* reason, const char* data, int len,
 	else
 		r = make_intrusive<StringVal>(reason);
 
-	const auto& tval = arg_tag ? arg_tag.AsVal() : tag.AsVal();
-	event_mgr.Enqueue(analyzer_violation, ConnVal(), tval, val_mgr->Count(id), std::move(r));
+	event_mgr.Enqueue(analyzer_violation, ConnVal(), arg_tag.AsVal(), val_mgr->Count(id),
+	                  std::move(r));
+	}
+
+void Analyzer::AnalyzerViolation(const char* reason, const char* data, int len, zeek::Tag arg_tag)
+	{
+	const auto& effective_tag = arg_tag ? arg_tag : tag;
+
+	++analyzer_violations;
+
+	if ( analyzer_violations > BifConst::max_analyzer_violations )
+		{
+		if ( analyzer_violations == BifConst::max_analyzer_violations + 1 )
+			Weird("too_many_analyzer_violations");
+
+		return;
+		}
+
+	if ( analyzer_violation_info )
+		EnqueueAnalyzerViolationInfo(reason, data, len, effective_tag);
+
+	if ( analyzer_violation )
+		EnqueueAnalyzerViolation(reason, data, len, effective_tag);
 	}
 
 void Analyzer::AddTimer(analyzer_timer_func timer, double t, bool do_expire,
@@ -861,3 +910,41 @@ void SupportAnalyzer::ForwardUndelivered(uint64_t seq, int len, bool is_orig)
 	}
 
 	} // namespace zeek::analyzer
+
+TEST_SUITE("Analyzer management")
+	{
+	TEST_CASE("Re-add analyzer after removal")
+		{
+		// This test tries to reactivate an analyzer which was previously removed.
+		// It's a regression test for #2801.
+		REQUIRE(zeek::analyzer_mgr);
+
+		zeek::Packet p;
+		zeek::ConnTuple t;
+		auto conn = std::make_unique<zeek::Connection>(zeek::detail::ConnKey(t), 0, &t, 0, &p);
+		auto* tcp = new zeek::packet_analysis::TCP::TCPSessionAdapter(conn.get());
+		conn->SetSessionAdapter(tcp, nullptr);
+
+		auto a = zeek::analyzer_mgr->InstantiateAnalyzer("SSH", conn.get());
+		REQUIRE(a);
+		auto b1 = zeek::analyzer_mgr->InstantiateAnalyzer("IMAP", a->Conn());
+		REQUIRE(b1);
+
+		tcp->AddChildAnalyzer(a);
+		a->AddChildAnalyzer(b1);
+
+		CHECK(conn->FindAnalyzer("SSH"));
+		CHECK(conn->FindAnalyzer("IMAP"));
+
+		a->RemoveChildAnalyzer(b1);
+
+		CHECK(! conn->FindAnalyzer("IMAP"));
+
+		auto b2 = zeek::analyzer_mgr->InstantiateAnalyzer("IMAP", a->Conn());
+		REQUIRE(b2);
+
+		REQUIRE(a->AddChildAnalyzer(b2));
+		CHECK(conn->FindAnalyzer("IMAP"));
+		conn->Done();
+		}
+	}
