@@ -3,6 +3,7 @@
 #pragma once
 
 #include <sys/types.h> // for u_char
+#include <unistd.h>
 
 extern "C"
 	{
@@ -28,9 +29,9 @@ protected:
 	void Close() override;
 	bool ExtractNextPacket(Packet* pkt) override;
 	void DoneWithPacket() override;
-	bool PrecompileFilter(int index, const std::string& filter) override;
 	bool SetFilter(int index) override;
 	void Statistics(Stats* stats) override;
+
 	detail::BPF_Program* CompileFilter(const std::string& filter) override;
 
 private:
