@@ -52,10 +52,6 @@ void Dispatcher::Register(uint32_t identifier, AnalyzerPtr analyzer)
 		}
 
 	int64_t index = identifier - lowest_identifier;
-	if ( table[index] != nullptr )
-		reporter->InternalWarning("Overwriting packet analyzer mapping %#8" PRIx64 " => %s with %s",
-		                          index + lowest_identifier, table[index]->GetAnalyzerName(),
-		                          analyzer->GetAnalyzerName());
 	table[index] = std::move(analyzer);
 	}
 
