@@ -787,6 +787,11 @@ void TCPSessionAdapter::SetPartialStatus(analyzer::tcp::TCP_Flags flags, bool is
 		}
 	}
 
+void TCPSessionAdapter::SetFirstPacketSeen(bool is_orig)
+	{
+	first_packet_seen |= (is_orig ? ORIG : RESP);
+	}
+
 void TCPSessionAdapter::UpdateInactiveState(double t, analyzer::tcp::TCP_Endpoint* endpoint,
                                             analyzer::tcp::TCP_Endpoint* peer, uint32_t base_seq,
                                             uint32_t ack_seq, int len, bool is_orig,
@@ -829,6 +834,7 @@ void TCPSessionAdapter::UpdateInactiveState(double t, analyzer::tcp::TCP_Endpoin
 				is_partial = 0;
 				Conn()->FlipRoles();
 				peer->SetState(analyzer::tcp::TCP_ENDPOINT_ESTABLISHED);
+				SetFirstPacketSeen(true);
 				}
 
 			else
@@ -908,11 +914,12 @@ void TCPSessionAdapter::UpdateInactiveState(double t, analyzer::tcp::TCP_Endpoin
 		else if ( flags.ACK() && peer->state == analyzer::tcp::TCP_ENDPOINT_ESTABLISHED )
 			{
 			// No SYN packet from originator but SYN/ACK from
-			// responder, and now a pure ACK. Problably means we
+			// responder, and now a pure ACK. Probably means we
 			// just missed that initial SYN. Let's not treat it
 			// as partial and instead establish the connection.
 			endpoint->SetState(analyzer::tcp::TCP_ENDPOINT_ESTABLISHED);
 			is_partial = 0;
+			SetFirstPacketSeen(is_orig);
 			}
 
 		else
@@ -1162,6 +1169,9 @@ void TCPSessionAdapter::FlipRoles()
 	orig = tmp_ep;
 	orig->is_orig = ! orig->is_orig;
 	resp->is_orig = ! resp->is_orig;
+	first_packet_seen = ((first_packet_seen & ORIG) ? RESP : 0) |
+	                    ((first_packet_seen & RESP) ? ORIG : 0);
+	is_partial = 0; // resetting, it may be re-established later
 	}
 
 void TCPSessionAdapter::UpdateConnVal(RecordVal* conn_val)
@@ -1602,7 +1612,7 @@ void TCPSessionAdapter::AddExtraAnalyzers(Connection* conn)
 	static zeek::Tag analyzer_connsize = analyzer_mgr->GetComponentTag("CONNSIZE");
 	static zeek::Tag analyzer_tcpstats = analyzer_mgr->GetComponentTag("TCPSTATS");
 
-	// We have to decide whether to reassamble the stream.
+	// We have to decide whether to reassemble the stream.
 	// We turn it on right away if we already have an app-layer
 	// analyzer, reassemble_first_packets is true, or the user
 	// asks us to do so.  In all other cases, reassembly may
@@ -1791,8 +1801,8 @@ int TCPSessionAdapter::ParseTCPOptions(const struct tcphdr* tcp, bool is_orig)
 					// timestamps
 					if ( length == 10 )
 						{
-						auto send = ntohl(*reinterpret_cast<const uint32_t*>(o + 2));
-						auto echo = ntohl(*reinterpret_cast<const uint32_t*>(o + 6));
+						uint32_t send = ntohl(*reinterpret_cast<const uint32_t*>(o + 2));
+						uint32_t echo = ntohl(*reinterpret_cast<const uint32_t*>(o + 6));
 						option_record->Assign(6, send);
 						option_record->Assign(7, echo);
 						}
@@ -1809,7 +1819,7 @@ int TCPSessionAdapter::ParseTCPOptions(const struct tcphdr* tcp, bool is_orig)
 						{
 						auto rate = o[2];
 						auto ttl_diff = o[3];
-						auto qs_nonce = ntohl(*reinterpret_cast<const uint32_t*>(o + 4));
+						uint32_t qs_nonce = ntohl(*reinterpret_cast<const uint32_t*>(o + 4));
 						option_record->Assign(8, rate);
 						option_record->Assign(9, ttl_diff);
 						option_record->Assign(10, qs_nonce);
