@@ -3,8 +3,10 @@
 #include "zeek/plugin/Manager.h"
 
 #include <dirent.h>
+#ifndef _MSC_VER
 #include <dlfcn.h>
 #include <glob.h>
+#endif
 #include <sys/stat.h>
 #include <cerrno>
 #include <climits> // for PATH_MAX
@@ -56,13 +58,13 @@ void Manager::SearchDynamicPlugins(const std::string& dir)
 	if ( dir.empty() )
 		return;
 
-	if ( dir.find(':') != string::npos )
+	if ( dir.find(path_list_separator) != string::npos )
 		{
 		// Split at ":".
 		std::stringstream s(dir);
 		std::string d;
 
-		while ( std::getline(s, d, ':') )
+		while ( std::getline(s, d, path_list_separator[0]) )
 			SearchDynamicPlugins(d);
 
 		return;
@@ -74,20 +76,23 @@ void Manager::SearchDynamicPlugins(const std::string& dir)
 		return;
 		}
 
-	char canon_path[PATH_MAX];
-	if ( ! realpath(dir.data(), canon_path) )
+	std::error_code ec;
+	auto canon = filesystem::canonical(dir, ec);
+	if ( ec )
 		{
-		DBG_LOG(DBG_PLUGINS, "skip dynamic plugin search in %s, realpath failed: %s", dir.data(),
-		        strerror(errno));
+		DBG_LOG(DBG_PLUGINS, "skip dynamic plugin search in %s, making path canonical failed: %s",
+		        dir.data(), ec.message().c_str());
 		return;
 		}
 
+	std::string canon_path = canon.string();
+
 	if ( searched_dirs.count(canon_path) )
 		return;
 
 	searched_dirs.emplace(canon_path);
 
-	// Check if it's a plugin dirctory.
+	// Check if it's a plugin directory.
 
 	const std::string magic = dir + "/__bro_plugin__";
 
@@ -121,7 +126,7 @@ void Manager::SearchDynamicPlugins(const std::string& dir)
 		return;
 		}
 
-	// No plugin here, traverse subirectories.
+	// No plugin here, traverse subdirectories.
 
 	DIR* d = opendir(dir.c_str());
 
@@ -160,10 +165,16 @@ void Manager::SearchDynamicPlugins(const std::string& dir)
 bool Manager::ActivateDynamicPluginInternal(const std::string& name, bool ok_if_not_found,
                                             std::vector<std::string>* errors)
 	{
+// Loading dynamic plugins is not currently supported on Windows platform.
+#ifdef _MSC_VER
+	return false;
+#else
 	errors->clear(); // caller should pass it in empty, but just to be sure
 
 	dynamic_plugin_map::iterator m = dynamic_plugins.find(util::strtolower(name));
 
+	plugin_list* all_plugins = Manager::ActivePluginsInternal();
+
 	if ( m == dynamic_plugins.end() )
 		{
 		if ( ok_if_not_found )
@@ -172,8 +183,6 @@ bool Manager::ActivateDynamicPluginInternal(const std::string& name, bool ok_if_
 		// Check if it's a static built-in plugin; they are always
 		// active, so just ignore. Not the most efficient way, but
 		// this should be rare to begin with.
-		plugin_list* all_plugins = Manager::ActivePluginsInternal();
-
 		for ( const auto& p : *all_plugins )
 			{
 			if ( p->Name() == name )
@@ -195,6 +204,23 @@ bool Manager::ActivateDynamicPluginInternal(const std::string& name, bool ok_if_
 
 	DBG_LOG(DBG_PLUGINS, "Activating plugin %s", name.c_str());
 
+	// If there's a plugin with the same name already, report an error and let
+	// the user do the conflict resolution.
+	auto lower_name = util::strtolower(name);
+	for ( const auto& p : *all_plugins )
+		{
+		if ( util::strtolower(p->Name()) == lower_name )
+			{
+			auto v = p->Version();
+			auto error = util::fmt(
+				"dynamic plugin %s from directory %s conflicts with %s plugin %s (%d.%d.%d)",
+				name.c_str(), dir.c_str(), p->DynamicPlugin() ? "dynamic" : "built-in",
+				p->Name().c_str(), v.major, v.minor, v.patch);
+			errors->push_back(error);
+			return false;
+			}
+		}
+
 	// Load shared libraries.
 
 	string dypattern = dir + "/lib/*." + HOST_ARCHITECTURE + DYNAMIC_PLUGIN_SUFFIX;
@@ -309,6 +335,7 @@ bool Manager::ActivateDynamicPluginInternal(const std::string& name, bool ok_if_
 	m->second.clear();
 
 	return true;
+#endif
 	}
 
 void Manager::ActivateDynamicPlugin(const std::string& name)
@@ -329,14 +356,21 @@ void Manager::ActivateDynamicPlugins(bool all)
 
 	// Activate plugins that were specifically requested.
 	for ( const auto& x : requested_plugins )
-		plugins_to_activate.emplace(x, false);
+		{
+		if ( ! x.empty() )
+			plugins_to_activate.emplace(x, false);
+		}
 
 	// Activate plugins that our environment tells us to.
 	vector<string> p;
-	util::tokenize_string(util::zeek_plugin_activate(), ",", &p);
+	std::string plugin_activate = util::zeek_plugin_activate();
+	if ( ! plugin_activate.empty() )
+		{
+		util::tokenize_string(util::zeek_plugin_activate(), ",", &p);
 
-	for ( const auto& x : p )
-		plugins_to_activate.emplace(x, true);
+		for ( const auto& x : p )
+			plugins_to_activate.emplace(x, true);
+		}
 
 	if ( all )
 		{
@@ -596,7 +630,7 @@ Plugin* Manager::LookupPluginByPath(std::string_view _path)
 static bool hook_cmp(std::pair<int, Plugin*> a, std::pair<int, Plugin*> b)
 	{
 	if ( a.first == b.first )
-		return util::strtolower(a.second->Name()) < util::strtolower(a.second->Name());
+		return util::strtolower(a.second->Name()) < util::strtolower(b.second->Name());
 
 	// Reverse sort.
 	return a.first > b.first;
@@ -894,32 +928,48 @@ void Manager::HookBroObjDtor(void* obj) const
 	if ( HavePluginForHook(META_HOOK_PRE) )
 		{
 		args.push_back(HookArgument(obj));
+#ifdef __GNUC__
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 		MetaHookPre(HOOK_BRO_OBJ_DTOR, args);
+#ifdef __GNUC__
 #pragma GCC diagnostic pop
+#endif
 		}
 
+#ifdef __GNUC__
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 	hook_list* l = hooks[HOOK_BRO_OBJ_DTOR];
+#ifdef __GNUC__
 #pragma GCC diagnostic pop
+#endif
 
 	if ( l )
 		for ( hook_list::iterator i = l->begin(); i != l->end(); ++i )
 			{
 			Plugin* p = (*i).second;
+#ifdef __GNUC__
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 			p->HookBroObjDtor(obj);
+#ifdef __GNUC__
 #pragma GCC diagnostic pop
+#endif
 			}
 
 	if ( HavePluginForHook(META_HOOK_POST) )
+#ifdef __GNUC__
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 		MetaHookPost(HOOK_BRO_OBJ_DTOR, args, HookArgument());
+#ifdef __GNUC__
 #pragma GCC diagnostic pop
+#endif
 	}
 
 void Manager::HookObjDtor(void* obj) const
