@@ -28,7 +28,7 @@ void CPP_IndexedInits<T>::Generate(InitsManager* im, std::vector<EnumValPtr>& iv
 	{
 	auto& e_type = im->Types(init_vals[0]);
 	int val = init_vals[1];
-	ivec[offset] = make_enum__CPP(e_type, val);
+	ivec[offset] = zeek::detail::make_enum__CPP(e_type, val);
 	}
 
 template <class T>
@@ -114,10 +114,14 @@ void CPP_IndexedInits<T>::Generate(InitsManager* im, std::vector<TableValPtr>& i
 	auto iv_it = init_vals.begin();
 	auto iv_end = init_vals.end();
 	auto t = *(iv_it++);
+	auto attrs = *(iv_it++);
 
 	auto tt = cast_intrusive<TableType>(im->Types(t));
 	auto tv = make_intrusive<TableVal>(tt);
 
+	if ( attrs >= 0 )
+		tv->SetAttrs(im->Attributes(attrs));
+
 	while ( iv_it != iv_end )
 		{
 		auto index = im->ConstVals(*(iv_it++));
@@ -166,12 +170,20 @@ void CPP_IndexedInits<T>::Generate(InitsManager* im, std::vector<AttrPtr>& ivec,
 	auto tag = static_cast<AttrTag>(init_vals[0]);
 	auto ae_tag = static_cast<AttrExprType>(init_vals[1]);
 
+	if ( ae_tag == AE_NONE )
+		{
+		ivec[offset] = make_intrusive<Attr>(tag);
+		return;
+		}
+
 	ExprPtr e;
 	auto e_arg = init_vals[2];
 
 	switch ( ae_tag )
 		{
 		case AE_NONE:
+			// Shouldn't happen, per test above.
+			ASSERT(0);
 			break;
 
 		case AE_CONST:
@@ -377,6 +389,8 @@ TypePtr CPP_TypeInits::BuildTypeList(InitsManager* im, ValElemVec& init_vals, in
 	while ( iv_it != iv_end )
 		tl->Append(im->Types(*(iv_it++)));
 
+	tl->CheckPure();
+
 	return tl;
 	}
 
@@ -482,14 +496,35 @@ int CPP_EnumMapping::ComputeOffset(InitsManager* im) const
 void CPP_GlobalInit::Generate(InitsManager* im, std::vector<void*>& /* inits_vec */,
                               int /* offset */) const
 	{
-	global = lookup_global__CPP(name, im->Types(type), exported);
+	auto& t = im->Types(type);
+	global = lookup_global__CPP(name, t, exported);
 
-	if ( ! global->HasVal() && val >= 0 )
+	if ( ! global->HasVal() )
 		{
-		global->SetVal(im->ConstVals(val));
-		if ( attrs >= 0 )
-			global->SetAttrs(im->Attributes(attrs));
+		if ( val >= 0 )
+			// Have explicit initialization value.
+			global->SetVal(im->ConstVals(val));
+
+		else if ( t->Tag() == TYPE_FUNC && ! func_with_no_val )
+			{
+			// Create a matching value so that this global can
+			// be used in other initializations.  The code here
+			// mirrors that in activate_bodies__CPP().
+			auto fn = global->Name();
+			auto ft = cast_intrusive<FuncType>(t);
+
+			vector<StmtPtr> no_bodies;
+			vector<int> no_priorities;
+
+			auto sf = make_intrusive<ScriptFunc>(fn, ft, no_bodies, no_priorities);
+
+			auto v = make_intrusive<FuncVal>(move(sf));
+			global->SetVal(v);
+			}
 		}
+
+	if ( attrs >= 0 )
+		global->SetAttrs(im->Attributes(attrs));
 	}
 
 void generate_indices_set(int* inits, std::vector<std::vector<int>>& indices_set)
