@@ -157,7 +157,7 @@ SubNetValPtr addr_mask__CPP(const IPAddr& a, uint32_t mask)
 	return make_intrusive<SubNetVal>(a, mask);
 	}
 
-// Helper function for reporting invalidation of interators.
+// Helper function for reporting invalidation of iterators.
 static void check_iterators__CPP(bool invalid)
 	{
 	if ( invalid )
@@ -208,7 +208,7 @@ void remove_element__CPP(TableValPtr aggr, ListValPtr indices)
 // A helper function that takes a parallel vectors of attribute tags
 // and values and returns a collective AttributesPtr corresponding to
 // those instantiated attributes.  For attributes that don't have
-// associated expressions, the correspoinding value should be nil.
+// associated expressions, the corresponding value should be nil.
 static AttributesPtr build_attrs__CPP(vector<int> attr_tags, vector<ValPtr> attr_vals)
 	{
 	vector<AttrPtr> attrs;
@@ -259,6 +259,11 @@ TableValPtr table_constructor__CPP(vector<ValPtr> indices, vector<ValPtr> vals,
 	return aggr;
 	}
 
+void assign_attrs__CPP(IDPtr id, std::vector<int> attr_tags, std::vector<ValPtr> attr_vals)
+	{
+	id->SetAttrs(build_attrs__CPP(move(attr_tags), move(attr_vals)));
+	}
+
 RecordValPtr record_constructor__CPP(vector<ValPtr> vals, RecordTypePtr t)
 	{
 	auto rv = make_intrusive<RecordVal>(t);
