@@ -59,7 +59,8 @@ const FuncInfo* analyze_global_stmts(Stmt* stmts)
 	auto sc = current_scope();
 	std::vector<IDPtr> empty_inits;
 	StmtPtr stmts_p{NewRef{}, stmts};
-	global_stmts = make_intrusive<ScriptFunc>(id, stmts_p, empty_inits, sc->Length(), 0);
+	global_stmts = make_intrusive<ScriptFunc>(id);
+	global_stmts->AddBody(stmts_p, empty_inits, sc->Length());
 
 	funcs.emplace_back(global_stmts, sc, stmts_p, 0);
 
@@ -271,14 +272,14 @@ static void init_options()
 	check_env_opt("ZEEK_PROFILE", analysis_options.profile_ZAM);
 
 	// Compile-to-C++-related options.
-	check_env_opt("ZEEK_ADD_CPP", analysis_options.add_CPP);
 	check_env_opt("ZEEK_GEN_CPP", analysis_options.gen_CPP);
 	check_env_opt("ZEEK_GEN_STANDALONE_CPP", analysis_options.gen_standalone_CPP);
 	check_env_opt("ZEEK_COMPILE_ALL", analysis_options.compile_all);
 	check_env_opt("ZEEK_REPORT_CPP", analysis_options.report_CPP);
 	check_env_opt("ZEEK_USE_CPP", analysis_options.use_CPP);
+	check_env_opt("ZEEK_ALLOW_COND", analysis_options.allow_cond);
 
-	if ( analysis_options.gen_standalone_CPP || analysis_options.add_CPP )
+	if ( analysis_options.gen_standalone_CPP )
 		analysis_options.gen_CPP = true;
 
 	if ( analysis_options.gen_CPP )
@@ -287,6 +288,10 @@ static void init_options()
 	if ( analysis_options.use_CPP && generating_CPP )
 		reporter->FatalError("generating C++ incompatible with using C++");
 
+	if ( analysis_options.allow_cond && ! analysis_options.gen_standalone_CPP )
+		reporter->FatalError(
+			"\"-O allow-cond\" only relevant when also using \"-O gen-standalone-C++\"");
+
 	auto usage = getenv("ZEEK_USAGE_ISSUES");
 
 	if ( usage )
@@ -414,22 +419,16 @@ static void use_CPP()
 
 	if ( num_used == 0 )
 		reporter->FatalError("no C++ functions found to use");
-
-	// Now that we've loaded all of the compiled scripts
-	// relevant for the AST, activate standalone ones.
-	for ( auto cb : standalone_activations )
-		(*cb)();
 	}
 
 static void generate_CPP(std::unique_ptr<ProfileFuncs>& pfs)
 	{
 	const auto gen_name = CPP_dir + "CPP-gen.cc";
 
-	const bool add = analysis_options.add_CPP;
 	const bool standalone = analysis_options.gen_standalone_CPP;
 	const bool report = analysis_options.report_uncompilable;
 
-	CPPCompile cpp(funcs, *pfs, gen_name, add, standalone, report);
+	CPPCompile cpp(funcs, *pfs, gen_name, standalone, report);
 	}
 
 static void analyze_scripts_for_ZAM(std::unique_ptr<ProfileFuncs>& pfs)
@@ -522,13 +521,12 @@ static void analyze_scripts_for_ZAM(std::unique_ptr<ProfileFuncs>& pfs)
 
 void analyze_scripts(bool no_unused_warnings)
 	{
-	static bool did_init = false;
+	init_options();
 
-	if ( ! did_init )
-		{
-		init_options();
-		did_init = true;
-		}
+	// Any standalone compiled scripts have already been instantiated
+	// at this point, but may require post-loading-of-scripts finalization.
+	for ( auto cb : standalone_finalizations )
+		(*cb)();
 
 	std::unique_ptr<UsageAnalyzer> ua;
 	if ( ! no_unused_warnings )
