@@ -15,6 +15,7 @@
 #endif
 
 #include <libgen.h>
+#include <unistd.h>
 #include <array>
 #include <cinttypes>
 #include <cstdarg>
@@ -30,27 +31,31 @@
 #ifdef TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <ctime>
-#else
-#ifdef HAVE_SYS_TIME_H
+#elif defined(HAVE_SYS_TIME_H)
 #include <sys/time.h>
 #else
 #include <ctime>
 #endif
-#endif
 
 #ifdef DEBUG
 
 #include <cassert>
 
+#ifdef ASSERT
+#undef ASSERT
+#endif
 #define ASSERT(x) assert(x)
-#define DEBUG_MSG(x...) fprintf(stderr, x)
+#define DEBUG_MSG(...) fprintf(stderr, __VA_ARGS__)
 #define DEBUG_fputs fputs
 
 #else
+#ifdef MSTCPIP_ASSERT_UNDEFINED
+#undef ASSERT
+#endif
 
 #define ASSERT(x)
-#define DEBUG_MSG(x...)
-#define DEBUG_fputs(x...)
+#define DEBUG_MSG(...)
+#define DEBUG_fputs(...)
 
 #endif
 
@@ -60,9 +65,15 @@
 extern HeapLeakChecker* heap_checker;
 #endif
 
-#include <pthread.h>
+#include <stdint.h>
+
+extern "C"
+	{
+#include "zeek/3rdparty/modp_numtoa.h"
+	}
 
 #ifdef HAVE_LINUX
+#include <pthread.h>
 #include <sys/prctl.h>
 #endif
 
@@ -70,12 +81,24 @@ extern HeapLeakChecker* heap_checker;
 #include <pthread_np.h>
 #endif
 
-extern "C"
+#ifdef _MSC_VER
+#include <pthread.h>
+#include <filesystem>
+namespace zeek
 	{
-#include "zeek/3rdparty/modp_numtoa.h"
+namespace filesystem = std::filesystem;
 	}
-
+inline constexpr std::string_view path_list_separator = ";";
+#else
+// Expose ghc::filesystem as zeek::filesystem until we can
+// switch to std::filesystem on all platforms.
 #include "zeek/3rdparty/ghc/filesystem.hpp"
+namespace zeek
+	{
+namespace filesystem = ghc::filesystem;
+	}
+inline constexpr std::string_view path_list_separator = ":";
+#endif
 
 using zeek_int_t = int64_t;
 using zeek_uint_t = uint64_t;
@@ -86,7 +109,7 @@ using bro_uint_t [[deprecated("Remove in v6.1. Use zeek_uint_t.")]] = zeek_uint_
 extern char* strcasestr(const char* s, const char* find);
 #endif
 
-// This is used by the patricia code and so it remains outside of hte namespace.
+// This is used by the patricia code and so it remains outside of the namespace.
 extern "C" void out_of_memory(const char* where);
 
 namespace zeek
@@ -95,10 +118,6 @@ namespace zeek
 class ODesc;
 class RecordVal;
 
-// Expose ghc::filesystem as zeek::filesystem until we can
-// switch to std::filesystem.
-namespace filesystem = ghc::filesystem;
-
 namespace util
 	{
 namespace detail
@@ -186,7 +205,7 @@ using SourceID = std::uintptr_t;
 constexpr SourceID SOURCE_LOCAL = 0;
 
 // TODO: This is a temporary marker to flag events coming in via Broker.
-// Those are remote events but we don't have any further peer informationa
+// Those are remote events but we don't have any further peer information
 // available for them (as the old communication code would have). Once we
 // remove RemoteSerializer, we can turn the SourceID into a simple boolean
 // indicating whether it's a local or remote event.
@@ -261,7 +280,7 @@ void terminate_processing();
 
 // Sets the current status of the Zeek process to the given string.
 // If the option --status-file has been set, this is written into
-// the the corresponding file.  Otherwise, the function is a no-op.
+// the corresponding file.  Otherwise, the function is a no-op.
 void set_processing_status(const char* status, const char* reason);
 
 // Renames the given file to a new temporary name, and opens a new file with
@@ -318,7 +337,7 @@ std::vector<std::string>* tokenize_string(std::string_view input, std::string_vi
 std::vector<std::string_view> tokenize_string(std::string_view input, const char delim) noexcept;
 
 extern char* copy_string(const char* s);
-extern int streq(const char* s1, const char* s2);
+extern bool streq(const char* s1, const char* s2);
 extern bool starts_with(std::string_view s, std::string_view beginning);
 extern bool ends_with(std::string_view s, std::string_view ending);
 
